@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -28,7 +28,7 @@
  * Message queues, writing to a UDP stack.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #include "common.h"
@@ -147,12 +147,23 @@ mq_udp_attach_metadata(pmsg_t *mb, const gnet_host_t *to)
 /**
  * Create new message queue capable of holding `maxsize' bytes, and
  * owned by the supplied node.
+ *
+ * @param maxsize		the overall sum of message size that can be held
+ * @param n				the network node to which the message queue is attached
+ * @oaram nd			the top of the TX stack to use to send out messages
+ * @param uops			user-defined operations
  */
 mqueue_t *
-mq_udp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
+mq_udp_make(int maxsize,
+	gnutella_node_t *n, struct txdriver *nd, const struct mq_uops *uops)
 {
 	mqueue_t *q;
 
+	node_check(n);
+	tx_check(nd);
+	g_assert(uops != NULL);
+	g_assert(maxsize > 0);
+
 	WALLOC0(q);
 
 	q->magic = MQ_MAGIC;
@@ -164,6 +175,7 @@ mq_udp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
 	q->qwait = slist_new();
 	q->ops = &mq_udp_ops;
 	q->cops = mq_get_cops();
+	q->uops = uops;
 	q->debug = GNET_PROPERTY_PTR(mq_udp_debug);
 
 	tx_srv_register(nd, mq_udp_service, q);
@@ -179,7 +191,7 @@ mq_udp_service(void *data)
 {
 	mqueue_t *q = data;
 	int r;
-	GList *l;
+	plist_t *l;
 	unsigned dropped = 0;
 
 	mq_check(q, 0);
@@ -266,8 +278,6 @@ void
 mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 {
 	size_t size;
-	char *mbs;
-	uint8 function;
 	pmsg_t *mbe = NULL;		/* Extended message with destination info */
 	bool error = FALSE;
 
@@ -329,10 +339,8 @@ again:
 	}
 	q->putq_entered++;
 
-	mbs = pmsg_start(mb);
-	function = gmsg_function(mbs);
-
-	gnet_stats_count_queued(q->node, function, mbs, size);
+	if (q->uops->msg_queued != NULL)
+		q->uops->msg_queued(q->node, mb);
 
 	/*
 	 * If queue is empty, attempt a write immediatly.
@@ -344,7 +352,8 @@ again:
 		if (pmsg_check(mb, q)) {
 			written = tx_sendto(q->tx_drv, mb, to);
 		} else {
-			gnet_stats_count_flowc(mbs, FALSE);
+			if (q->uops->msg_flowc != NULL)
+				q->uops->msg_flowc(q->node, mb);
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 			written = (ssize_t) -1;
 		}
