@@ -37,8 +37,9 @@
 #include "common.h"
 
 #include "share.h"
-#include "extensions.h"
+
 #include "downloads.h"
+#include "extensions.h"
 #include "fileinfo.h"
 #include "ggep_type.h"
 #include "gmsg.h"
@@ -55,6 +56,7 @@
 #include "search.h"
 #include "settings.h"
 #include "spam.h"
+#include "tth_cache.h"
 #include "upload_stats.h"
 #include "uploads.h"
 
@@ -286,6 +288,10 @@ static hikset_t *sha1_to_share;		/* Marked thread-safe */
 #define D	SEARCH_DOC_TYPE
 #define I	SEARCH_IMG_TYPE
 #define U	(SEARCH_WIN_TYPE | SEARCH_UNIX_TYPE)
+#define T	SEARCH_TORRENT_TYPE
+
+#define TXT	SEARCH_G2_TEXT_TYPE
+#define ROM	SEARCH_G2_ROM_TYPE
 
 /**
  * This table encodes for each known MIME type the searchable media type bits
@@ -295,7 +301,7 @@ static hikset_t *sha1_to_share;		/* Marked thread-safe */
  * of the searchable bits.  This typically indicates a file containing
  * either application-specific data (e.g. an MP3 playlist), a programming
  * language (e.g. a C header file), or binary-specific data (e.g. a ROM
- * image).
+ * image, although these are now flagged as ROM, for the benefit of G2 queries).
  */
 static struct {
 	enum mime_type type;
@@ -305,7 +311,7 @@ static struct {
 	{ MIME_TYPE_APPLICATION_7Z,						U },
 	{ MIME_TYPE_APPLICATION_ACE,					U },
 	{ MIME_TYPE_APPLICATION_ANDROID_PACKAGE,		U },
-	{ MIME_TYPE_APPLICATION_BITTORRENT,				D },
+	{ MIME_TYPE_APPLICATION_BITTORRENT,				T },
 	{ MIME_TYPE_APPLICATION_BROADBAND_EBOOK,		D },
 	{ MIME_TYPE_APPLICATION_BZIP2,					U },
 	{ MIME_TYPE_APPLICATION_COMPILED_HTML_HELP,		D },
@@ -315,8 +321,8 @@ static struct {
 	{ MIME_TYPE_APPLICATION_DOSEXEC,				U },
 	{ MIME_TYPE_APPLICATION_EPUB,					D },
 	{ MIME_TYPE_APPLICATION_EXCEL,					0 },
-	{ MIME_TYPE_APPLICATION_GAMEBOY_ROM,			0 },
-	{ MIME_TYPE_APPLICATION_GENESIS_ROM,			0 },
+	{ MIME_TYPE_APPLICATION_GAMEBOY_ROM,			ROM },
+	{ MIME_TYPE_APPLICATION_GENESIS_ROM,			ROM },
 	{ MIME_TYPE_APPLICATION_GZIP,					U },
 	{ MIME_TYPE_APPLICATION_IPHONE_APP,				0 },
 	{ MIME_TYPE_APPLICATION_ISO9660,				U },
@@ -330,8 +336,8 @@ static struct {
 	{ MIME_TYPE_APPLICATION_MSWORD,					D },
 	{ MIME_TYPE_APPLICATION_MS_READER,				D },
 	{ MIME_TYPE_APPLICATION_MS_SHORTCUT,			0 },
-	{ MIME_TYPE_APPLICATION_N64_ROM,				0 },
-	{ MIME_TYPE_APPLICATION_NES_ROM,				0 },
+	{ MIME_TYPE_APPLICATION_N64_ROM,				ROM },
+	{ MIME_TYPE_APPLICATION_NES_ROM,				ROM },
 	{ MIME_TYPE_APPLICATION_OBJECT,					0 },
 	{ MIME_TYPE_APPLICATION_OGG,					A },
 	{ MIME_TYPE_APPLICATION_OPEN_PACKAGING_FORMAT,	U },
@@ -345,7 +351,7 @@ static struct {
 	{ MIME_TYPE_APPLICATION_SHAR,					U },
 	{ MIME_TYPE_APPLICATION_SHOCKWAVE_FLASH,		V },
 	{ MIME_TYPE_APPLICATION_SIT,					U },
-	{ MIME_TYPE_APPLICATION_SNES_ROM,				0 },
+	{ MIME_TYPE_APPLICATION_SNES_ROM,				ROM },
 	{ MIME_TYPE_APPLICATION_TAR,					U },
 	{ MIME_TYPE_APPLICATION_TEX,					D },
 	{ MIME_TYPE_APPLICATION_TEXINFO,				D },
@@ -389,7 +395,7 @@ static struct {
 	{ MIME_TYPE_TEXT_LATEX,							D },
 	{ MIME_TYPE_TEXT_LILYPOND,						D },
 	{ MIME_TYPE_TEXT_PERL,							0 },
-	{ MIME_TYPE_TEXT_PLAIN,							D },
+	{ MIME_TYPE_TEXT_PLAIN,							D|TXT },
 	{ MIME_TYPE_TEXT_PYTHON,						0 },
 	{ MIME_TYPE_TEXT_RTF,							D },
 	{ MIME_TYPE_TEXT_XHTML,							D },
@@ -410,6 +416,9 @@ static struct {
 #undef D
 #undef I
 #undef U
+#undef T
+#undef TXT
+#undef ROM
 
 /**
  * Hash table yielding the media type flags from a MIME type.
@@ -1366,6 +1375,60 @@ directory_is_unshareable(const char *dir)
 
 	return FALSE;	/* No objection */
 }
+/**
+ * Generate a set of TTH hashes belonging to shared files.
+ * This set can be disposed of by share_tthset_free().
+ *
+ * @return newly allocate set of shared TTH (atoms).
+ */
+hset_t *
+share_tthset_get(void)
+{
+	uint64 n;
+	shared_file_t **sfp;
+	hset_t *set;
+
+	set = hset_create(HASH_KEY_FIXED, TTH_RAW_SIZE);
+
+	SHARED_LIBFILE_LOCK;
+
+	n = shared_libfile.files_scanned;
+	sfp = shared_libfile.file_table;
+
+	while (n-- != 0) {
+		shared_file_t *sf = *sfp++;
+
+		shared_file_check(sf);
+
+		if (sf->tth != NULL && !hset_contains(set, sf->tth)) {
+			const struct tth *tth = atom_tth_get(sf->tth);
+			hset_insert(set, tth);
+		}
+	}
+
+	SHARED_LIBFILE_UNLOCK;
+
+	return set;
+}
+
+static void
+share_tthset_keyfree(const void *data, void *udata)
+{
+	const struct tth *tth = data;
+
+	(void) udata;
+	atom_tth_free(tth);
+}
+
+/**
+ * Free set created by share_tthset_get().
+ */
+void
+share_tthset_free(hset_t *set)
+{
+	hset_foreach(set, share_tthset_keyfree, NULL);
+	hset_free_null(&set);
+}
 
 enum recursive_scan_magic { RECURSIVE_SCAN_MAGIC = 0x16926d87U };
 
@@ -1791,7 +1854,6 @@ recursive_scan_readdir(struct recursive_scan *ctx)
 
 finish:
 	HFREE_NULL(fullpath);
-	dir_entry_filename(NULL);	/* release memory */
 }
 
 /**
@@ -1825,8 +1887,19 @@ recursive_scan_done(struct bgtask *bt, void *data, bgstatus_t status, void *arg)
 	if (THREAD_MAIN == share_thread_id) {
 		struct share_thread_vars *v = &share_thread_vars;
 
+		/*
+		 * Taking the lock is not really necessary here because if we run
+		 * in the main thread, everything is serialized, so there cannot be
+		 * any concurrent access on these variables.  For uniformity though...
+		 *		--RAM, 2015-03-06
+		 */
+
+		spinlock(&v->lock);
+
 		if (bt == v->task)
 			v->task = NULL;
+
+		spinunlock(&v->lock);
 	}
 }
 
@@ -2417,6 +2490,25 @@ recursive_scan_step_request_sha1(struct bgtask *bt, void *data, int ticks)
 	return BGR_NEXT;
 }
 
+static bgret_t
+recursive_scan_step_tth_cache_cleanup(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+
+	recursive_scan_check(ctx);
+	(void) bt;
+	(void) ticks;
+
+	/*
+	 * Now that we have the library of shared files all setup, see whether
+	 * there are entries in the TTH cache that are no longer required
+	 * because their TTH is no longer associated with a shared file.
+	 */
+
+	tth_cache_cleanup();
+	return BGR_NEXT;
+}
+
 /**
  * First step, intalling signal handler to trap task cancel.
  */
@@ -2735,9 +2827,10 @@ share_rescan_create_task(bgsched_t *bs)
 		recursive_scan_step_build_sorted_table,
 		recursive_scan_step_install_shared,
 		recursive_scan_step_request_sha1,
+		recursive_scan_step_tth_cache_cleanup,
 
 		/*
-		 * Remains steps identical to the ones listed in
+		 * The following group of steps is identical to the ones listed in
 		 * share_update_qrp_create_task().
 		 */
 
@@ -3010,12 +3103,10 @@ share_thread_create(void)
 	 */
 
 	r = thread_create(share_thread_main, barrier_refcnt_inc(b),
-			THREAD_F_DETACH | THREAD_F_NO_CANCEL | THREAD_F_NO_POOL,
+			THREAD_F_DETACH | THREAD_F_NO_CANCEL |
+				THREAD_F_NO_POOL | THREAD_F_PANIC,
 			THREAD_STACK_MIN);
 
-	if (-1 == r)
-		s_error("%s(): cannot create library thread: %m", G_STRFUNC);
-
 	barrier_wait(b);		/* Wait for thread to initialize */
 	barrier_free_null(&b);
 
