@@ -39,14 +39,16 @@
 
 #include "if/gnet_property_priv.h"
 
-#include "lib/misc.h"
-#include "lib/sha1.h"
 #include "lib/base64.h"
-#include "lib/glib-missing.h"
 #include "lib/crc.h"
+#include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/misc.h"
 #include "lib/random.h"
+#include "lib/sha1.h"
 #include "lib/stacktrace.h"
 #include "lib/tm.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 #define TOKEN_CLOCK_SKEW	3600		/**< +/- 1 hour */
@@ -408,6 +410,130 @@ static const char *keys_101_1[] = {
 	"2547 4731 9fbd 4c83 e58a aa5b f401 c0e8",
 };
 
+static const char *keys_101_2[] = {
+	"5475 43e8 a2ca 1404 f2d9 df56 2feb 70eb",
+	"53ab ae62 458a e08a c3fc 49e0 f94e 7aff",
+	"6493 d9ac 61e7 9a3a e1ab abc7 95f6 2072",
+	"6f79 2aac e10c ebdb 1e9f e9c3 0df9 4b2b",
+	"78db 1395 8bea 63a3 933f 1760 7860 d480",
+	"e4d2 40a9 a4a3 fc73 5dfa 3f56 15f7 254a",
+	"70f6 3795 1a4e 7e32 74bc bfc6 366c b889",
+	"fc10 b6c8 1f52 3134 531b 62e0 150b 6101",
+	"02ab c0b8 1bd2 6cea 3fd0 c191 ecc4 0262",
+	"e41b 3732 d070 ae4b ac4b c36b 176f 0a03",
+	"0de1 e1bb 4dd0 81a1 5365 cf5c d358 d7ba",
+	"c0bc b0fd 762b 773a a85b 21c8 2af0 f797",
+	"405d 4997 01db e436 ba6e 2222 cc28 9b6d",
+	"2ab5 245e aaee d65d 92f0 c65a b08c d636",
+	"ff11 3970 246c ee6e a00d 4ebd 54c0 ca31",
+	"c5a9 63a7 28da b2d2 8d3b a63e 0957 cf7a",
+	"d4f0 a55a 0ea6 100c 7062 cd93 a886 e802",
+	"49c9 76e6 f907 5b02 46de 70f9 236e 828d",
+	"d1f6 91e2 7f5a e8d8 2ebe 754e 2041 31cc",
+	"4467 80bb 0874 6e60 d3a2 d12f 37cd eb8b",
+	"9a5a 6ddb 440b f584 8a7d fdd0 a026 8c05",
+	"635d 9f51 d228 70bf bdba 995f f360 4171",
+	"e7a7 b9d3 a02c b0e6 a66d 9aef aeb4 187d",
+	"989c 1c07 ebc0 a635 4a4f 2bf5 305c cb0f",
+	"790d 7a2c c570 3856 8d79 6909 3479 85de",
+	"373b 7f3e c464 de1e a6c9 a0b0 c810 20b0",
+	"de00 19a9 8674 2c20 789f f688 d721 2a23",
+	"e351 701b 3cac f6d3 37b3 7052 2b64 f64c",
+};
+
+static const char *keys_101_3[] = {
+	"be45 6d2e b759 55b8 0616 85b4 6efe 829e",
+	"4003 5f01 b4a2 3026 23ef 6bbf d221 955c",
+	"8e58 304c 6aa7 7ffb 0a5a 89f7 3384 3735",
+	"1176 4ef2 e01d 5256 0d98 92e4 3bdd 6d59",
+	"d2e3 d4db 4752 263f bfc5 ecc5 2268 ee26",
+	"b349 5c28 6217 57da 0c18 6372 fa89 99f7",
+	"7f87 fce2 7d76 5b5b 863a 2bbd 1db9 b3bf",
+	"b161 dca9 e137 c304 fbae 867e b816 68b3",
+	"d8ac b802 e14d ad55 14fa 2c91 6b01 fd40",
+	"a10e c052 1e95 627c 70b9 0975 38d9 82c5",
+	"338e 9ad5 4949 c9ca a87b 7ee1 abe9 780b",
+	"9179 88e8 82ed e94e 2bc9 dd04 a6ca fae7",
+	"856b b3a8 2f28 9b4d 7e96 4675 4d42 2096",
+	"8b1f eac3 1c28 023c 757d 539c 493c 7935",
+	"b222 4f2b 6d93 a501 0f8c 3e95 614d 84cf",
+	"b7de 80b5 a08d d9e5 c9fa 530a 96ee c1cb",
+	"7c52 4279 bcfb c2fb 65eb 7a75 13c3 0b7a",
+	"c6b9 14cc 0738 b907 372b 5f16 e6c6 46d6",
+	"a392 0388 295a 0eba 3546 2c0d 3fdf 8fb8",
+	"9d2a 0362 60fb 3881 ef6e 3261 094f 2cdd",
+	"f1ea 482e 8b57 d3b6 8d71 15fb 9e2b 4d93",
+	"cf84 15c5 00c8 7232 e35b 707c e21e be62",
+	"a79f 219f adda bf80 e00b cef9 5991 6674",
+	"cdf9 0894 103e 9f32 fe12 3779 671b c4ee",
+	"3553 c1f0 2c52 ed5d 8933 28b2 2e82 5bdf",
+	"16ef 4da0 76b0 10eb cb8e 4c73 dead a6c6",
+	"c4c9 c157 2f92 8df7 bf67 70cb e8a8 1e41",
+	"ec54 de4b 6290 152e 3e4f 3a7e 225b d9bf",
+};
+
+static const char *keys_101_4[] = {
+	"58fc e374 b0bb 6513 e300 bbc2 d6fe c15d",
+	"aa0c 2ec1 4704 284e 6342 12cc 05aa 678f",
+	"ec54 b643 d0d7 631e b76c 0d71 4c5b 7476",
+	"766e 1c8f a71c 03d6 c611 6e29 b0e5 97fe",
+	"e4b0 0719 1e4a b4d4 5f91 fa95 91c4 a9e2",
+	"ad25 ef5e d04a bb02 5c9d dfeb 3d0b a088",
+	"7058 eea1 6f08 e792 b904 87ed 6702 968d",
+	"b1a7 1ba5 0eee da59 c434 36a7 904c d43f",
+	"c44f 63dd e8eb d380 2859 f4d0 2a5e 5500",
+	"e2e2 4b89 1a6b c2d2 9e1b 22eb de4e 41a4",
+	"0fd1 6eb7 fd7c 6b4b c021 b5b5 c913 0553",
+	"a41a e6ef 8da3 d109 f253 1a10 139d 39ce",
+	"721e da21 b10c 43a6 ecce ab57 6b73 e231",
+	"14a3 0b58 ca81 053a 8fe0 cce4 c8c6 7f1b",
+	"2be9 a5e3 a3ad 1990 058f d116 d3eb 9f5c",
+	"622b c7d9 247d f8a8 fb4d 963a 8797 ee18",
+	"b12d 9e6a eb8e 5df8 0b3c 57f6 52c6 5b43",
+	"6233 2f0a 8e46 c3a9 526b 6b56 92e3 3c12",
+	"8db5 5af8 9527 ea2b 1f22 8a70 bcc8 621a",
+	"e55a 223d 23f7 ed62 c381 3f7c b151 9d28",
+	"92b5 626f d69f c011 ade4 ac57 d8ea 2e37",
+	"7881 c4b7 cf45 8484 e735 f6f0 71a8 1ffe",
+	"7788 755d aeb5 f875 be16 6587 1dd0 3e01",
+	"fe56 c9c3 b194 da09 bbf8 25be 5f2b e2da",
+	"c2f3 f1ff 580b efa4 1e0c 30dd 7478 fc20",
+	"73b1 97b0 dc00 f43d 8327 9c56 0369 51a8",
+	"60a0 3f09 e110 1812 3dde 75e8 6402 f5f0",
+	"c0c9 b75a df8f 784b 2ff3 7a25 f175 03f5",
+};
+
+static const char *keys_101_5[] = {
+	"2b70 9d92 a693 86c1 a5b3 a55b c55e 48a9",
+	"e118 e30a f9ef 1a31 5e22 c8cb 9cc4 d3f8",
+	"75af 1193 e9b3 64d7 6150 2531 6f07 e23a",
+	"2920 8768 b211 a247 2f78 7456 dd24 9849",
+	"25fc a295 fcb2 3147 13d4 4210 ee2b 45f5",
+	"e76d 1a8a 929c 4382 8aee 4e56 fe65 d9dc",
+	"d4a8 405c 9f07 09f8 ae6b e8e4 783e 0328",
+	"6b3a fdbd 50ec f9f4 1ee1 7e49 df0b 49e0",
+	"d11a a613 be62 2a55 ed82 5a28 d4e9 0763",
+	"45f8 cb52 ddad 22e7 f9b7 0124 d0e8 bbe3",
+	"0ed3 8f13 62e1 b68c 83ea 9824 d919 8776",
+	"f331 1edb c576 dd92 b12d 7b6f 2064 6b36",
+	"5db1 a911 77b0 9c20 3350 5001 8856 0b9c",
+	"08d6 b62b 8c35 a7d4 dfba a18e faee efee",
+	"d7f4 4df4 dfdb 7ef3 2dff bc45 614c 527e",
+	"66dd 661d eef6 3c4d cc36 464e 9c13 a847",
+	"c2b6 2207 773a 6d01 8d84 ad4c 5ae0 07c4",
+	"4b58 8da5 3fa5 128f 5d73 b019 cb0b 46ca",
+	"6aab 3555 605b 5b93 e093 3db9 bc1e 631a",
+	"9b10 ec1d ac5b 5e39 6e97 33e3 3a02 1c04",
+	"95b5 b829 3533 4897 c2b3 24a3 0c31 3354",
+	"7310 5672 717c 10f1 6dd1 90e5 c533 4b80",
+	"b0a7 5c33 01dc 042e fd9e 4f00 ca27 4426",
+	"b450 430b 9e63 a101 e05f dfb6 7e45 10cf",
+	"ab14 6c74 2781 1e6f b558 0970 c7ed 50dc",
+	"fe89 9f1d 8a2e 10d6 1cc1 1094 8177 c3fc",
+	"b1c3 3046 2613 c585 a1fa 8701 6573 a37a",
+	"ca0e 8b49 2c8b 961a 9244 bcff 122d be09",
+};
+
 #define KEYS(x)		keys_ ## x, G_N_ELEMENTS(keys_ ## x)
 
 /**
@@ -418,6 +544,7 @@ struct tokkey {
 	const char **keys;	/**< Keys to use */
 	uint count;			/**< Amount of keys defined */
 } token_keys[] = {
+	/* maj min PL  tag lvl #  timestamp     key array          ISO date   */
 	{ { 0, 97, 0, '\0', 0, 0, 1310940000 }, KEYS(097_0)	},	/* 2011-07-18 */
 	{ { 0, 97, 1, '\0', 0, 0, GIT_SWITCH }, KEYS(097_1)	},	/* 2011-09-11 */
 	{ { 0, 98, 0, '\0', 0, 0, 1323558000 }, KEYS(098_0)	},	/* 2011-12-11 */
@@ -429,6 +556,10 @@ struct tokkey {
 	{ { 1, 0,  1, '\0', 0, 0, 1388444400 }, KEYS(100_1) },	/* 2013-12-31 */
 	{ { 1, 1,  0, '\0', 0, 0, 1404252000 }, KEYS(101_0) },	/* 2014-07-02 */
 	{ { 1, 1,  1, '\0', 0, 0, 1409608800 }, KEYS(101_1) },	/* 2014-09-02 */
+	{ { 1, 1,  2, '\0', 0, 0, 1426892400 }, KEYS(101_2) },	/* 2015-03-21 */
+	{ { 1, 1,  3, '\0', 0, 0, 1428444000 }, KEYS(101_3) },	/* 2015-04-08 */
+	{ { 1, 1,  4, '\0', 0, 0, 1441749600 }, KEYS(101_4) },	/* 2015-09-09 */
+	{ { 1, 1,  5, '\0', 0, 0, 1444255200 }, KEYS(101_5) },	/* 2015-10-08 */
 };
 
 #undef KEYS
@@ -745,7 +876,7 @@ tok_generate(time_t now, const char *version)
 	ZERO(&lvlbase64);
 	base64_encode_into(lvldigest, 2 * lvlsize, lvlbase64, LEVEL_BASE64_SIZE);
 
-	return g_strconcat(token, "; ", lvlbase64, (void *) 0);
+	return h_strconcat(token, "; ", lvlbase64, (void *) 0);
 }
 
 /**
@@ -779,7 +910,7 @@ tok_version(void)
 
 	last_generated = now;
 
-	G_FREE_NULL(toklevel);
+	HFREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_string);
 
 	return NOT_LEAKING(toklevel);
@@ -810,7 +941,7 @@ tok_short_version(void)
 
 	last_generated = now;
 
-	G_FREE_NULL(toklevel);
+	HFREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_short_string);
 
 	return NOT_LEAKING(toklevel);
