@@ -37,6 +37,7 @@
 #include "cq.h"
 
 #include "atoms.h"
+#include "buf.h"
 #include "elist.h"
 #include "entropy.h"
 #include "hashing.h"		/* For integer_hash_fast() */
@@ -676,7 +677,7 @@ cq_insert(cqueue_t *cq, int delay, cq_service_t fn, void *arg)
  *
  * This is the preferred way of zeroing a reference to the event because it
  * checks that this is indeed the event being dispatched.  It can only be
- * called from a callout callback, where the queue is already locked.
+ * called from a callout callback.
  *
  * It can be used on normal or extended events (registrations from a foreign
  * thread).  It must only be called once per event, but client code should not
@@ -715,6 +716,9 @@ cq_zero(cqueue_t *cq, cevent_t **ev_ptr)
 	 * was returned to the calling thread to be stored in "ev_ptr".
 	 */
 
+	g_assert_log(cq->cq_call != NULL,
+		"%s() can only be called once on a given event", G_STRFUNC);
+
 	g_assert_log(ev == cq->cq_call || (NULL == ev && cq->cq_call_extended),
 		"%s() not called on current event: %p points to ev=%p, current is %s%p",
 		G_STRFUNC, ev_ptr, ev, cq->cq_call_extended ? "foreign " : "",
@@ -1932,10 +1936,11 @@ cq_main_coverage(int old_ticks)
 const char *
 cq_time_to_string(cq_time_t t)
 {
-	static char buf[UINT64_DEC_BUFLEN];
+	buf_t *b = buf_private(G_STRFUNC, UINT64_DEC_BUFLEN);
+	char *p = buf_data(b);
 
-	uint64_to_string_buf(t, buf, sizeof buf);
-	return buf;
+	uint64_to_string_buf(t, p, buf_size(b));
+	return p;
 }
 
 /***
@@ -2023,11 +2028,8 @@ cq_global_init(void)
 	if (thread_main_is_blockable()) {
 		callout_thread = TRUE;
 		callout_queue->cq_stid = thread_create(cq_thread_main, NULL,
-			THREAD_F_DETACH | THREAD_F_NO_POOL, CALLOUT_THREAD_STACK);
-		if (-1U == callout_queue->cq_stid) {
-			s_minierror("%s(): cannot launch callout queue thread: %m",
-				G_STRFUNC);
-		}
+			THREAD_F_DETACH | THREAD_F_NO_POOL | THREAD_F_PANIC,
+			CALLOUT_THREAD_STACK);
 	} else {
 		callout_timer_id = g_timeout_add(CALLOUT_PERIOD,
 			cq_heartbeat_trampoline, callout_queue);
