@@ -45,10 +45,10 @@
 
 #include "common.h"
 
-#include "sdbm/sdbm.h"
-
 #include "dbmap.h"
+
 #include "bstr.h"
+#include "pslist.h"
 #include "debug.h"
 #include "map.h"
 #include "pmsg.h"
@@ -56,6 +56,8 @@
 #include "unsigned.h"			/* For size_is_non_negative() */
 #include "walloc.h"
 
+#include "sdbm/sdbm.h"
+
 #include "override.h"			/* Must be the last header included */
 
 enum dbmap_magic { DBMAP_MAGIC = 0x5890dc4fU };
@@ -212,7 +214,7 @@ dbmap_sdbm_retrieve_superblock(DBM *sdbm, struct dbmap_superblock *block)
 	}
 
 	if (version > DBMAP_SUPERKEY_VERSION) {
-		g_warning("SDBM \"%s\": superblock more recent "
+		s_warning("SDBM \"%s\": superblock more recent "
 			"(version %u, can only understand up to version %u)",
 			sdbm_name(sdbm), version, DBMAP_SUPERKEY_VERSION);
 	}
@@ -251,7 +253,7 @@ dbmap_sdbm_strip_superblock(DBM *sdbm)
 	if (0 == sdbm_delete(sdbm, key))
 		return TRUE;
 
-	g_warning("SDBM \"%s\": cannot strip superblock: %m", sdbm_name(sdbm));
+	s_warning("SDBM \"%s\": cannot strip superblock: %m", sdbm_name(sdbm));
 
 	return FALSE;
 }
@@ -311,10 +313,9 @@ dbmap_sdbm_count_keys(dbmap_t *dm, bool expect_superblock)
 
 	if (dbmap_sdbm_retrieve_superblock(sdbm, &sblock)) {
 		if (common_dbg) {
-			g_debug("SDBM \"%s\": superblock has %u key%s%s, "
+			s_debug("SDBM \"%s\": superblock has %u key%s%s, "
 				"last check done %s ago",
-				sdbm_name(sdbm), (unsigned) sblock.count,
-				1 == sblock.count ? "" : "s",
+				sdbm_name(sdbm), (unsigned) sblock.count, plural(sblock.count),
 				(sblock.flags & DBMAP_SF_KEYCHECK) ?
 					" (keycheck required)" : "",
 				compact_time(delta_time(tm_time(), sblock.last_check)));
@@ -325,14 +326,14 @@ dbmap_sdbm_count_keys(dbmap_t *dm, bool expect_superblock)
 			time_delta_t d = delta_time(tm_time(), sblock.last_check);
 			if (d >= DBMAP_SDBM_CHECK_PERIOD) {
 				if (common_dbg) {
-					g_debug("SDBM \"%s\": %s since last check, verifying keys",
+					s_debug("SDBM \"%s\": %s since last check, verifying keys",
 						sdbm_name(sdbm), compact_time(d));
 				}
 				goto check_db;
 			}
 			if (sblock.flags & DBMAP_SF_KEYCHECK) {
 				if (common_dbg) {
-					g_debug("SDBM \"%s\": verifying keys as requested",
+					s_debug("SDBM \"%s\": verifying keys as requested",
 						sdbm_name(sdbm));
 				}
 				goto check_db;
@@ -341,13 +342,13 @@ dbmap_sdbm_count_keys(dbmap_t *dm, bool expect_superblock)
 			return sblock.count;
 		} else {
 			if (common_dbg) {
-				g_debug("SDBM \"%s\": unexpected superblock, checking keys",
+				s_debug("SDBM \"%s\": unexpected superblock, checking keys",
 					sdbm_name(sdbm));
 			}
 		}
 	} else if (expect_superblock) {
 		if (common_dbg) {
-			g_debug("SDBM \"%s\": no superblock, counting and checking keys",
+			s_debug("SDBM \"%s\": no superblock, counting and checking keys",
 				sdbm_name(sdbm));
 		}
 	}
@@ -364,7 +365,7 @@ check_db:
 	dm->u.s.last_check = tm_time();
 
 	if (sdbm_error(sdbm)) {
-		g_warning("SDBM \"%s\": I/O error after key counting, clearing",
+		s_warning("SDBM \"%s\": I/O error after key counting, clearing",
 			sdbm_name(sdbm));
 		sdbm_clearerr(sdbm);
 	}
@@ -737,16 +738,16 @@ dbmap_remove(dbmap_t *dm, const void *key)
 			} else {
 				if G_UNLIKELY(0 == dm->count) {
 					if (dm->validated) {
-						g_critical("DBMAP on sdbm \"%s\": BUG: "
+						s_critical("DBMAP on sdbm \"%s\": BUG: "
 							"sdbm_delete() worked but we had no key tracked",
 							sdbm_name(dm->u.s.sdbm));
 					} else {
-						g_warning("DBMAP on sdbm \"%s\": "
+						s_warning("DBMAP on sdbm \"%s\": "
 							"key count inconsistency, validating database",
 							sdbm_name(dm->u.s.sdbm));
 					}
 					dm->count = dbmap_sdbm_count_keys(dm, FALSE);
-					g_warning("DBMAP on sdbm \"%s\": "
+					s_warning("DBMAP on sdbm \"%s\": "
 						"key count reset to %zu after counting",
 						sdbm_name(dm->u.s.sdbm), dm->count);
 				} else {
@@ -928,7 +929,7 @@ dbmap_destroy(dbmap_t *dm)
 }
 
 struct insert_ctx {
-	GSList *sl;
+	pslist_t *sl;
 	const dbmap_t *dm;
 };
 
@@ -944,17 +945,17 @@ insert_key(void *key, void *unused_value, void *u)
 	(void) unused_value;
 
 	kdup = wcopy(key, dbmap_keylen(ctx->dm, key));
-	ctx->sl = g_slist_prepend(ctx->sl, kdup);
+	ctx->sl = pslist_prepend(ctx->sl, kdup);
 }
 
 /**
  * Snapshot all the constant-width keys, returning them in a singly linked list.
  * To free the returned keys, use the dbmap_free_all_keys() helper.
  */
-GSList *
+pslist_t *
 dbmap_all_keys(const dbmap_t *dm)
 {
-	GSList *sl = NULL;
+	pslist_t *sl = NULL;
 
 	dbmap_check(dm);
 
@@ -986,7 +987,7 @@ dbmap_all_keys(const dbmap_t *dm)
 					continue;		/* Invalid key, corrupted file? */
 
 				kdup = wcopy(key.dptr, key.dsize);
-				sl = g_slist_prepend(sl, kdup);
+				sl = pslist_prepend(sl, kdup);
 			}
 			dbmap_sdbm_error_check(dm);
 		}
@@ -1002,14 +1003,14 @@ dbmap_all_keys(const dbmap_t *dm)
  * Helper routine to free list and keys returned by dbmap_all_keys().
  */
 void
-dbmap_free_all_keys(const dbmap_t *dm, GSList *keys)
+dbmap_free_all_keys(const dbmap_t *dm, pslist_t *keys)
 {
-	GSList *sl;
+	pslist_t *sl;
 
-	GM_SLIST_FOREACH(keys, sl) {
+	PSLIST_FOREACH(keys, sl) {
 		wfree(sl->data, dbmap_keylen(dm, sl->data));
 	}
-	g_slist_free(keys);
+	pslist_free(keys);
 }
 
 /**
@@ -1134,10 +1135,9 @@ dbmap_foreach(const dbmap_t *dm, dbmap_cb_t cb, void *arg)
 			if (!dbmap_sdbm_error_check(dm))
 				dbmap_reset_count(dm, count);
 			if (invalid || unreadable) {
-				g_warning("DBMAP on sdbm \"%s\": found %zu invalid key%s and "
+				s_warning("DBMAP on sdbm \"%s\": found %zu invalid key%s and "
 					"%zu unreadable",
-					sdbm_name(sdbm), invalid, 1 == invalid ? "" : "s",
-					unreadable);
+					sdbm_name(sdbm), invalid, plural(invalid), unreadable);
 			}
 		}
 		break;
@@ -1206,7 +1206,7 @@ dbmap_foreach_remove(const dbmap_t *dm, dbmap_cbr_t cbr, void *arg)
 							deleted++;
 						} else {
 							errors++;
-							g_warning("DBMAP on sdbm \"%s\": "
+							s_warning("DBMAP on sdbm \"%s\": "
 								"key deletion error: %m", sdbm_name(sdbm));
 						}
 					}
@@ -1218,10 +1218,10 @@ dbmap_foreach_remove(const dbmap_t *dm, dbmap_cbr_t cbr, void *arg)
 			dbmap_sdbm_error_check(dm);
 			dbmap_reset_count(dm, count);
 			if (invalid || errors || unreadable) {
-				g_warning("DBMAP on sdbm \"%s\": found %zu invalid key%s, "
+				s_warning("DBMAP on sdbm \"%s\": found %zu invalid key%s, "
 					"%zu key deletion error%s, %zu unreadable",
-					sdbm_name(sdbm), invalid, 1 == invalid ? "" : "s",
-					errors, 1 == errors ? "" : "s", unreadable);
+					sdbm_name(sdbm), invalid, plural(invalid),
+					errors, plural(errors), unreadable);
 			}
 		}
 		break;
@@ -1267,7 +1267,7 @@ dbmap_store(dbmap_t *dm, const char *base, bool inplace)
 			return ok;
 		}
 
-		g_warning("SDBM \"%s\": cannot store superblock: %m",
+		s_warning("SDBM \"%s\": cannot store superblock: %m",
 			sdbm_name(dm->u.s.sdbm));
 
 		/* FALL THROUGH */
@@ -1280,7 +1280,7 @@ dbmap_store(dbmap_t *dm, const char *base, bool inplace)
 		O_CREAT | O_TRUNC | O_RDWR, S_IRUSR | S_IWUSR);
 
 	if (!ndm) {
-		g_warning("SDBM \"%s\": cannot store to %s: %m",
+		s_warning("SDBM \"%s\": cannot store to %s: %m",
 			sdbm_name(dm->u.s.sdbm), base);
 		return FALSE;
 	}
@@ -1288,7 +1288,7 @@ dbmap_store(dbmap_t *dm, const char *base, bool inplace)
 	dbmap_foreach(dm, dbmap_store_entry, ndm);
 
 	if (sdbm_error(ndm->u.s.sdbm)) {
-		g_warning("SDBM \"%s\": cannot store to %s: errors during dump",
+		s_warning("SDBM \"%s\": cannot store to %s: errors during dump",
 			sdbm_name(dm->u.s.sdbm), base);
 		ok = FALSE;
 		goto done;
