@@ -67,7 +67,9 @@
 #include "common.h"
 
 #include "etree.h"
+
 #include "unsigned.h"
+#include "xslist.h"
 
 #include "override.h"			/* Must be the last header included */
 
@@ -106,6 +108,20 @@ etree_init(etree_t *tree, bool extended, size_t offset)
 	tree->count = 0;
 }
 
+/**
+ * Set initial root of empty tree.
+ */
+void
+etree_set_root(etree_t *tree, const void *root)
+{
+	etree_check(tree);
+	g_assert(NULL == tree->root);
+
+	tree->root = ptr_add_offset(deconstify_pointer(root), tree->offset);
+	if (NULL == tree->root->child)
+		tree->count = 1;
+}
+
 /**
  * Discard tree, making the tree object invalid.
  *
@@ -351,6 +367,18 @@ etree_detach(etree_t *tree, void *item)
 	}
 }
 
+/**
+ * Increment tree count if known and added node is a leaf.
+ */
+static inline void
+etree_count_update_added(etree_t *tree, const node_t *node)
+{
+	if (NULL == node->child && 0 != tree->count)
+		tree->count++;			/* Adding leaf node */
+	else
+		tree->count = 0;		/* Tree count is now unknown */
+}
+
 /**
  * Append child to parent.
  *
@@ -396,7 +424,8 @@ etree_append_child(etree_t *tree, void *parent, void *child)
 	}
 
 	cn->parent = pn;
-	tree->count = 0;		/* Tree count is now unknown */
+
+	etree_count_update_added(tree, cn);
 }
 
 /**
@@ -430,7 +459,7 @@ etree_prepend_child(etree_t *tree, void *parent, void *child)
 		}
 	}
 
-	tree->count = 0;		/* Tree count is now unknown */
+	etree_count_update_added(tree, cn);
 }
 
 /**
@@ -493,7 +522,7 @@ etree_add_right_sibling(etree_t *tree, void *node, void *item)
 		px->last_child = i;
 	}
 
-	tree->count = 0;		/* Tree count is now unknown */
+	etree_count_update_added(tree, i);
 }
 
 /**
@@ -533,7 +562,104 @@ etree_add_left_sibling(etree_t *tree, void *node, void *item)
 		p->sibling = i;
 	}
 
-	tree->count = 0;		/* Tree count is now unknown */
+	etree_count_update_added(tree, i);
+}
+
+/**
+ * Recursively sort all the children of each tree node.
+ *
+ * This does NOT sort the tree so that traversal is in a given order implied
+ * by the comparison routine.  It only deals with the immediate children of
+ * each parent node.
+ *
+ * @param tree		the tree to sort
+ * @param root		the parent node whose children we want to sort
+ * @param cmp		the comparison routine for two items in the tree
+ * @param data		additional argument to supply to comparison routine
+ *
+ * @return amount of nodes visited.
+ */
+static size_t
+etree_sort_internal(etree_t *tree, node_t *root, cmp_data_fn_t cmp, void *data)
+{
+	void *child;
+	node_t *n, *next;
+	xslist_t siblings;
+	bool is_extended;
+	size_t visited;
+
+	etree_check(tree);
+
+	if (NULL == root->child)
+		return 1;
+
+	/*
+	 * Sort the siblings before iterating over them to recurse.
+	 *
+	 * We're going to treat the sibling list (one-way linked via n->sibling)
+	 * as an expanded single list whose link offset is the offset of the
+	 * ``sibling'' field in the node_t structure.
+	 */
+
+	child = ptr_add_offset(root->child, -tree->offset);
+	xslist_load(&siblings, child, tree->offset, offsetof(node_t, sibling));
+	xslist_sort_with_data(&siblings, cmp, data);
+	root->child = xslist_first(&siblings);
+
+	is_extended = etree_is_extended(tree);
+	visited = 1;
+
+	for (n = root->child; n != NULL; n = next) {
+		g_assert(root == n->parent);	/* Sorting did not alter structure */
+
+		next = n->sibling;
+		visited += etree_sort_internal(tree, n, cmp, data);
+
+		if G_UNLIKELY(is_extended && NULL == next) {
+			nodex_t *nx = (nodex_t *) root;
+			nx->last_child = n;
+		}
+	}
+
+	return visited;
+}
+
+/**
+ * Sort the children of each tree node with the item-comparison routine.
+ *
+ * @param tree	the tree whose children we want to sort
+ * @param cmp	the comparison routine to compare two items
+ */
+void
+etree_sort(etree_t *tree, cmp_fn_t cmp)
+{
+	etree_check(tree);
+
+	if (NULL == tree->root)
+		return;
+
+	tree->count =
+		etree_sort_internal(tree, tree->root, (cmp_data_fn_t) cmp, NULL);
+}
+
+/**
+ * Sort the children of each tree node with the item-comparison routine,
+ * which is extended to take an additional user-supplied context.
+ *
+ * @param tree	the tree whose children we want to sort
+ * @param cmp	the comparison routine to compare two items
+ * @param data	additional argument to supply to comparison routine
+ */
+void
+etree_sort_with_data(etree_t *tree, cmp_data_fn_t cmp, void *data)
+{
+	etree_check(tree);
+
+	if (NULL == tree->root)
+		return;
+
+	tree->count =
+		etree_sort_internal(tree, tree->root, (cmp_data_fn_t) cmp, data);
 }
 
 /**
@@ -565,7 +691,7 @@ etree_traverse_internal(const etree_t *tree, node_t *root,
 	unsigned flags, unsigned curdepth, unsigned mindepth, unsigned maxdepth,
 	match_fn_t enter, data_fn_t action, void *data)
 {
-	size_t visited = 0;
+	size_t visited = 1;		/* 1 for this node */
 	void *item;
 	bool actionable;
 	node_t *child;
@@ -579,7 +705,7 @@ etree_traverse_internal(const etree_t *tree, node_t *root,
 	 * Check whether we need to execute action on the node.
 	 */
 
-	child = root->child;	/* Save pointer in case "action" frees node */
+	child = root->child;
 
 	if (curdepth < mindepth)
 		actionable = FALSE;
@@ -620,7 +746,7 @@ etree_traverse_internal(const etree_t *tree, node_t *root,
 	if (actionable && (flags & ETREE_CALL_AFTER))
 		(*action)(item, data);		/* Can safely free node */
 
-	return visited + 1;		/* "+1" for this node */
+	return visited;
 }
 
 /**
