@@ -287,12 +287,10 @@ evq_init_once(void)
 	atomic_bool_set(&evq_run, TRUE);
 
 	evq_thread_id = thread_create(evq_thread_main, NULL,
-			THREAD_F_DETACH | THREAD_F_NO_CANCEL | THREAD_F_NO_POOL,
+			THREAD_F_DETACH | THREAD_F_NO_CANCEL |
+				THREAD_F_NO_POOL | THREAD_F_PANIC,
 			EVQ_STACK_SIZE);
 
-	if (-1U == evq_thread_id)
-		s_error("%s(): cannot create the event queue thread: %m", G_STRFUNC);
-
 	evq_fully_inited = TRUE;
 }
 
@@ -424,7 +422,7 @@ evq_event_discard(void *data, void *udata)
 
 	s_warning("%s(): discarding %s event %s(%p) for %s",
 		G_STRFUNC, what, stacktrace_function_name(eve->cb), eve->arg,
-		thread_name());
+		thread_id_name(eve->stid));
 
 	evq_event_free(eve);
 }
@@ -987,6 +985,10 @@ evq_cancel(evq_event_t **eve_ptr)
  * We do not make the ev_queue variable visible from the outside because
  * we need to call evq_notify() each time we add an event since the event
  * queue heartbeats are not regularily spaced.
+ *
+ * @param delay		delay (in ms) before triggering the event
+ * @param fn		the function to invoke when delay expired
+ * @param arg		argument to pass to function
  */
 cevent_t *
 evq_raw_insert(int delay, cq_service_t fn, void *arg)
@@ -1009,6 +1011,9 @@ evq_raw_insert(int delay, cq_service_t fn, void *arg)
  *
  * Use cq_idle_remove() directly to cancel it or have the event return FALSE
  * to stop periodic invocations..
+ *
+ * @param event		the function to invoke when idle
+ * @param arg		argument to pass to function
  */
 cidle_t *
 evq_raw_idle_add(cq_invoke_t event, void *arg)
@@ -1031,6 +1036,10 @@ evq_raw_idle_add(cq_invoke_t event, void *arg)
  *
  * Use cq_periodic_remove() directly to cancel it or have the event return
  * FALSE to stop periodic invocations..
+ *
+ * @param delay		period (in ms) between two event invocations
+ * @param event		the function to invoke periodically
+ * @param arg		argument to pass to function
  */
 cperiodic_t *
 evq_raw_periodic_add(int period, cq_invoke_t event, void *arg)
