@@ -68,12 +68,12 @@
 #include "hashtable.h"
 
 #include "atomic.h"
-#include "entropy.h"
 #include "hashing.h"
 #include "mutex.h"
 #include "omalloc.h"
 #include "once.h"
 #include "pow2.h"
+#include "rand31.h"
 #include "spinlock.h"
 #include "vmm.h"
 #include "xmalloc.h"
@@ -162,8 +162,13 @@ static once_flag_t hash_min_bins_computed;
 static G_GNUC_COLD void
 hash_offset_init_once(void)
 {
-	/* Don't allocate any memory, hence can't call arc4random() */
-	hash_offset[0] = entropy_minirand();
+	/*
+	 * Don't allocate any memory, hence can't call arc4random().
+	 * Also, on Windows, we cannot enter the POSIX layer by trying
+	 * to open files, hence cannot use entropy_minirand() either.
+	 */
+
+	hash_offset[0] = rand31_u32();
 
 	/*
 	 * Note that we leave hash_offset[1] set to 0: hash offsetting is not
@@ -281,23 +286,6 @@ hash_table_check(const hash_table_t *ht)
 	g_assert(ht->num_bins > 0 && ht->num_bins < SIZE_MAX / 2);
 }
 
-static inline unsigned
-hash_id_key(const void *key)
-{
-	/*
-	 * A naive direct use of the pointer has a much worse distribution,
-	 * e.g. only a quarter of the bins are used.
-	 */
-
-	return GOLDEN_RATIO_32 * pointer_to_ulong(key);
-}
-
-static inline bool
-hash_id_eq(const void *a, const void *b)
-{
-	return a == b;
-}
-
 static hash_item_t *
 hash_item_alloc(hash_table_t *ht, const void *key, const void *value)
 {
@@ -363,9 +351,9 @@ hash_table_new_intern(hash_table_t *ht,
 	ht->magic = HASHTABLE_MAGIC;
 	ht->num_held = 0;
 	ht->bin_fill = 0;
-	ht->hash = hash != NULL ? hash : hash_id_key;
-	ht->eq = eq != NULL ? eq : hash_id_eq;
-	ht->self_keys = booleanize(hash_id_eq == ht->eq);
+	ht->hash = hash != NULL ? hash : pointer_hash;
+	ht->eq = eq != NULL ? eq : pointer_eq;
+	ht->self_keys = booleanize(pointer_eq == ht->eq);
 
 	/*
 	 * If data space is not-NULL, then the table is going to be of fixed_size.
