@@ -49,6 +49,7 @@
 #include "host_addr.h"
 
 #include "ascii.h"
+#include "buf.h"
 #include "concat.h"
 #include "endian.h"
 #include "glib-missing.h"		/* For g_strlcpy() */
@@ -79,7 +80,7 @@ host_addr_hash(host_addr_t ha)
 		}
 		/* FALL THROUGH */
 	case NET_TYPE_IPV4:
-		return ha.net ^ integer_hash(host_addr_ipv4(ha));
+		return ha.net ^ integer_hash_fast(host_addr_ipv4(ha));
 	case NET_TYPE_LOCAL:
 	case NET_TYPE_NONE:
 		return ha.net;
@@ -114,6 +115,59 @@ host_addr_hash2(host_addr_t ha)
 	return (unsigned) -1;
 }
 
+/**
+ * Hashing of host_addr_t + port.
+ */
+unsigned
+host_addr_port_hash(host_addr_t ha, uint16 port)
+{
+	switch (ha.net) {
+	case NET_TYPE_IPV6:
+		{
+			host_addr_t ha_ipv4;
+
+			if (!host_addr_convert(ha, &ha_ipv4, NET_TYPE_IPV4))
+				return binary_hash(&ha.addr.ipv6[0], sizeof ha.addr.ipv6);
+			ha = ha_ipv4;
+		}
+		/* FALL THROUGH */
+	case NET_TYPE_IPV4:
+		return hashing_mix32(
+			ha.net + integer_hash_fast(host_addr_ipv4(ha)) + u16_hash(port));
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		return hashing_mix32(ha.net + u16_hash(port));
+	}
+	g_assert_not_reached();
+	return (unsigned) -1;
+}
+
+/**
+ * Alternate hashing of host_addr_t + port.
+ */
+unsigned
+host_addr_port_hash2(host_addr_t ha, uint16 port)
+{
+	switch (ha.net) {
+	case NET_TYPE_IPV6:
+		{
+			host_addr_t ha_ipv4;
+
+			if (!host_addr_convert(ha, &ha_ipv4, NET_TYPE_IPV4))
+				return binary_hash2(&ha.addr.ipv6[0], sizeof ha.addr.ipv6);
+			ha = ha_ipv4;
+		}
+		/* FALL THROUGH */
+	case NET_TYPE_IPV4:
+		return ha.net + integer_hash2(host_addr_ipv4(ha)) + u16_hash2(port);
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		return ha.net + u16_hash2(port);
+	}
+	g_assert_not_reached();
+	return (unsigned) -1;
+}
+
 /**
  * Check whether hosts are equal.
  */
@@ -568,12 +622,13 @@ host_addr_to_string_buf(const host_addr_t ha, char *dst, size_t size)
 const char *
 host_addr_to_string(const host_addr_t ha)
 {
-	static char buf[HOST_ADDR_BUFLEN];
-	size_t n;
+	buf_t *b = buf_private(G_STRFUNC, HOST_ADDR_BUFLEN);
+	char *p = buf_data(b);
+	size_t len, n = buf_size(b);
 
-	n = host_addr_to_string_buf(ha, buf, sizeof buf);
-	g_assert(n < sizeof buf);
-	return buf;
+	len = host_addr_to_string_buf(ha, p, n);
+	g_assert(len < n);
+	return p;
 }
 
 /**
@@ -582,12 +637,13 @@ host_addr_to_string(const host_addr_t ha)
 const char *
 host_addr_to_string2(const host_addr_t ha)
 {
-	static char buf[HOST_ADDR_BUFLEN];
-	size_t n;
+	buf_t *b = buf_private(G_STRFUNC, HOST_ADDR_BUFLEN);
+	char *p = buf_data(b);
+	size_t len, n = buf_size(b);
 
-	n = host_addr_to_string_buf(ha, buf, sizeof buf);
-	g_assert(n < sizeof buf);
-	return buf;
+	len = host_addr_to_string_buf(ha, p, n);
+	g_assert(len < n);
+	return p;
 }
 
 /**
@@ -640,23 +696,25 @@ host_addr_port_to_string_buf(const host_addr_t ha, uint16 port,
 const char *
 host_addr_port_to_string(const host_addr_t ha, uint16 port)
 {
-	static char buf[HOST_ADDR_PORT_BUFLEN];
-	size_t n;
+	buf_t *b = buf_private(G_STRFUNC, HOST_ADDR_PORT_BUFLEN);
+	char *p = buf_data(b);
+	size_t len, n = buf_size(b);
 
-	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
-	g_assert(n < sizeof buf);
-	return buf;
+	len = host_addr_port_to_string_buf(ha, port, p, n);
+	g_assert(len < n);
+	return p;
 }
 
 const char *
 host_addr_port_to_string2(const host_addr_t ha, uint16 port)
 {
-	static char buf[HOST_ADDR_PORT_BUFLEN];
-	size_t n;
+	buf_t *b = buf_private(G_STRFUNC, HOST_ADDR_PORT_BUFLEN);
+	char *p = buf_data(b);
+	size_t len, n = buf_size(b);
 
-	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
-	g_assert(n < sizeof buf);
-	return buf;
+	len = host_addr_port_to_string_buf(ha, port, p, n);
+	g_assert(len < n);
+	return p;
 }
 
 /**
@@ -709,28 +767,30 @@ host_port_addr_to_string_buf(uint16 port, const host_addr_t ha,
 const char *
 port_host_addr_to_string(uint16 port, const host_addr_t ha)
 {
-	static char buf[HOST_ADDR_PORT_BUFLEN];
-	size_t n;
+	buf_t *b = buf_private(G_STRFUNC, HOST_ADDR_PORT_BUFLEN);
+	char *p = buf_data(b);
+	size_t len, n = buf_size(b);
 
-	n = host_port_addr_to_string_buf(port, ha, buf, sizeof buf);
-	g_assert(n < sizeof buf);
-	return buf;
+	len = host_port_addr_to_string_buf(port, ha, p, n);
+	g_assert(len < n);
+	return p;
 }
 
 const char *
 host_port_to_string(const char *hostname, host_addr_t addr, uint16 port)
 {
-	static char buf[MAX_HOSTLEN + 32];
+	buf_t *b = buf_private(G_STRFUNC, MAX_HOSTLEN + HOST_ADDR_PORT_BUFLEN);
+	char *p = buf_data(b);
 
-	if (hostname) {
+	if (hostname != NULL) {
 		char port_buf[UINT32_DEC_BUFLEN];
 
 		uint32_to_string_buf(port, port_buf, sizeof port_buf);
-		concat_strings(buf, sizeof buf, hostname, ":", port_buf, (void *) 0);
+		concat_strings(p, buf_size(b), hostname, ":", port_buf, (void *) 0);
 	} else {
-		host_addr_port_to_string_buf(addr, port, buf, sizeof buf);
+		host_addr_port_to_string_buf(addr, port, p, buf_size(b));
 	}
-	return buf;
+	return p;
 }
 
 /**
@@ -1611,12 +1671,34 @@ packed_host_addr_unpack(const struct packed_host_addr paddr)
 	return zero_host_addr;
 }
 
-uint
-packed_host_addr_size(const struct packed_host_addr paddr)
+host_addr_t
+packed_host_addr_unpack_ptr(const struct packed_host_addr *paddr)
 {
-	switch (paddr.net) {
-	case NET_TYPE_IPV4:	 return 5;
-	case NET_TYPE_IPV6:  return 17;
+	g_assert(paddr != NULL);
+
+	switch (paddr->net) {
+	case NET_TYPE_IPV4:
+		return host_addr_peek_ipv4(&paddr->addr[0]);
+	case NET_TYPE_IPV6:
+		return host_addr_peek_ipv6(paddr->addr);
+	case NET_TYPE_LOCAL:
+		return local_host_addr;
+	case NET_TYPE_NONE:
+		return zero_host_addr;
+	}
+	g_assert_not_reached();
+	return zero_host_addr;
+}
+
+/**
+ * Significant size of a packed host_addr (serialization size).
+ */
+uint
+packed_host_addr_size_ptr(const struct packed_host_addr *paddr)
+{
+	switch (paddr->net) {
+	case NET_TYPE_IPV4:	 return 1 + 4;
+	case NET_TYPE_IPV6:  return 1 + 16;
 	case NET_TYPE_LOCAL: return 1;
 	case NET_TYPE_NONE:	 return 1;
 	}
@@ -1624,6 +1706,12 @@ packed_host_addr_size(const struct packed_host_addr paddr)
 	return 0;
 }
 
+uint
+packed_host_addr_size(const struct packed_host_addr paddr)
+{
+	return packed_host_addr_size_ptr(&paddr);
+}
+
 struct packed_host
 host_pack(const host_addr_t addr, uint16 port)
 {
@@ -1703,7 +1791,6 @@ packed_host_size_ptr(const struct packed_host *phost)
 	return 0;
 }
 
-
 /**
  * Significant size of a packed host (serialization size).
  */
@@ -1760,16 +1847,9 @@ packed_host_eq_func(const void *p, const void *q)
 void *
 walloc_packed_host(const host_addr_t addr, uint16 port)
 {
-	struct packed_host packed;
-	void *key;
-	uint len;
+	struct packed_host packed = host_pack(addr, port);
 
-	packed = host_pack(addr, port);
-	len = packed_host_size_ptr(&packed);
-	key = walloc(len);
-	memcpy(key, &packed, len);
-
-	return key;
+	return wcopy(&packed, packed_host_size_ptr(&packed));
 }
 
 /**
@@ -1785,4 +1865,79 @@ wfree_packed_host(void *key, void *unused_data)
 	wfree(key, packed_host_size_ptr(p));
 }
 
+/***
+ *** The following routines are meant to be used for hash tables indexed
+ *** by packed_host_addr structures, which are variable-sized entities.
+ ***/
+
+/**
+ * Hash a packed host_addr buffer (variable-sized).
+ */
+uint
+packed_host_addr_hash(const void *key)
+{
+	const struct packed_host_addr *p = key;
+	return binary_hash(key, packed_host_addr_size_ptr(p));
+}
+
+/**
+ * Alternate hash for a packed host_addr buffer (variable-sized).
+ */
+uint
+packed_host_addr_hash2(const void *key)
+{
+	const struct packed_host_addr *p = key;
+	return binary_hash2(key, packed_host_addr_size_ptr(p));
+}
+
+/**
+ * Compare two packed host_addr buffers (variable-sized).
+ */
+bool
+packed_host_addr_equal(const void *p, const void *q)
+{
+	const struct packed_host_addr *a = p, *b = q;
+	uint asize = packed_host_addr_size_ptr(a);
+	uint bsize = packed_host_addr_size_ptr(b);
+
+	if (asize != bsize)
+		return FALSE;
+
+	return 0 == memcmp(a, b, asize);
+}
+
+/**
+ * Allocate a packed_host_addr object from a host address.
+ */
+void *
+walloc_packed_host_addr(const host_addr_t addr)
+{
+	struct packed_host_addr packed = host_addr_pack(addr);
+
+	return wcopy(&packed, packed_host_addr_size_ptr(&packed));
+}
+
+/**
+ * Allocate a packed_host_addr object from a host address pointer.
+ */
+void *
+walloc_packed_host_addr_ptr(const void *paddr)
+{
+	const host_addr_t *addr = paddr;
+	struct packed_host_addr packed = host_addr_pack(*addr);
+
+	return wcopy(&packed, packed_host_addr_size_ptr(&packed));
+}
+
+/**
+ * Release packed_host_addr allocated via walloc_packed_host_addr().
+ */
+void
+wfree_packed_host_addr(void *pha)
+{
+	const struct packed_host_addr *p = pha;
+
+	wfree(pha, packed_host_addr_size_ptr(p));
+}
+
 /* vi: set ts=4 sw=4 cindent: */
