@@ -876,6 +876,11 @@ bin_to_hex_buf(const void *data, size_t len, char *dst, size_t size)
 size_t
 guid_to_string_buf(const guid_t *guid, char *dst, size_t size)
 {
+	if G_UNLIKELY(NULL == guid) {
+		/* Our constant string is 32-byte long */
+		return g_strlcpy(dst, "<------ null GUID pointer ----->", size);
+	}
+
 	return bin_to_hex_buf(guid->v, GUID_RAW_SIZE, dst, size);
 }
 
@@ -1132,7 +1137,7 @@ base32_to_guid(const char *base32, guid_t *guid)
 	g_assert(base32 != NULL);
 	g_assert(guid != NULL);
 
-	ret = base32_decode(guid->v, sizeof guid->v, base32, GUID_BASE32_SIZE);
+	ret = base32_decode(guid, sizeof *guid, base32, GUID_BASE32_SIZE);
 	return (size_t) 0 + GUID_RAW_SIZE == ret ? guid : NULL;
 }
 
@@ -1336,7 +1341,7 @@ base32_sha1(const char *base32)
 
 	g_assert(base32 != NULL);
 
-	len = base32_decode(s->data, sizeof s->data, base32, SHA1_BASE32_SIZE);
+	len = base32_decode(s, sizeof *s, base32, SHA1_BASE32_SIZE);
 	return SHA1_RAW_SIZE == len ? s : NULL;
 }
 
@@ -1377,7 +1382,7 @@ base32_tth(const char *base32)
 
 	g_assert(base32 != NULL);
 
-	len = base32_decode(t->data, sizeof t->data, base32, TTH_BASE32_SIZE);
+	len = base32_decode(t, sizeof *t, base32, TTH_BASE32_SIZE);
 	return TTH_RAW_SIZE == len ? t : NULL;
 }
 
@@ -2383,6 +2388,9 @@ symbolic_errno(int errnum)
 #endif
 #ifdef ENOTBLK
 	CASE(ENOTBLK);
+#endif
+#ifdef ESHUTDOWN
+	CASE(ESHUTDOWN);
 #endif
 	}
 #undef CASE
