@@ -209,7 +209,7 @@ bool is_symlink(const char *pathname);
 int is_same_file(const char *, const char *);
 
 /**
- * Tries to extrace the file mode from a struct dirent. Not all systems
+ * Tries to extract the file mode from a struct dirent. Not all systems
  * support this, in which case zero is returned. Types other than regular
  * files, directories and symlinks are ignored and gain a value of zero
  * as well.
@@ -238,13 +238,27 @@ dir_entry_mode(const struct dirent *dir_entry)
 }
 
 #ifndef MINGW32
-/**
- * @note NULL is allowed as parameter, see MINGW32 implementation.
- */
+/* There is a MINGW32 version defined for Windows in lib/mingw32.c */
 static inline const char *
 dir_entry_filename(const struct dirent *dir_entry)
 {
-	return NULL != dir_entry ? dir_entry->d_name : NULL;
+	g_assert(dir_entry != NULL);
+
+	return dir_entry->d_name;
+}
+
+/* There is a MINGW32 version defined for Windows in lib/mingw32.c */
+static inline size_t
+dir_entry_namelen(const struct dirent *dir_entry)
+{
+	g_assert(dir_entry != NULL);
+
+#ifdef HAS_DIRENT_D_NAMLEN
+	if G_LIKELY(dir_entry->d_namlen != 0)
+		return dir_entry->d_namlen;
+#endif	/* HAS_DIRENT_D_NAMLEN */
+
+	return strlen(dir_entry->d_name);
 }
 #endif	/* MINGW32 */
 
@@ -600,9 +614,6 @@ static inline bool
 is_temporary_error(int error)
 {
   switch (error) {
-#ifdef WSAEWOULDBLOCK
-  case WSAEWOULDBLOCK:
-#endif
   case EAGAIN:
 #if defined(EWOULDBLOCK) && EAGAIN != EWOULDBLOCK
   case EWOULDBLOCK:
