@@ -44,7 +44,7 @@
  * problem, only bad luck.
  *
  * To (slowly) generate strong random numbers, use entropy_random().
- * For (faster) strong random nubers, use arc4random().
+ * For (faster) strong random numbers, use arc4random().
  *
  * @author Raphael Manfredi
  * @date 2012
@@ -59,7 +59,6 @@
 #include "hashing.h"
 #include "log.h"
 #include "mempcpy.h"
-#include "pow2.h"
 #include "random.h"
 #include "spinlock.h"
 #include "stacktrace.h"
@@ -117,10 +116,14 @@ rand31_random_seed(void)
 
 	tm_precise_time(&now);		/* NOT tm_now_exact(): it is too early */
 	nsecs = now.tv_nsec;
+	seed = integer_hash_fast(now.tv_nsec) + integer_hash_fast(now.tv_sec);
 	cpu = tm_cputime(NULL, NULL);
 	tm_precise_time(&now);
 	nsecs += now.tv_nsec;
-	seed = (GOLDEN_RATIO_31 * getpid()) >> 1;
+	seed += integer_hash_fast(getpid());
+#ifdef HAS_GETPPID
+	seed += integer_hash_fast(getppid());
+#endif
 	seed += binary_hash(&now, sizeof now);
 	seed += binary_hash(&cpu, sizeof cpu);
 	seed += pointer_hash_fast(&now);
@@ -128,6 +131,7 @@ rand31_random_seed(void)
 	tm_precise_time(&now);
 	nsecs += now.tv_nsec;
 	seed += binary_hash(&now, sizeof now);
+	seed += integer_hash_fast(nsecs);
 	ZERO(&env);			/* Avoid uninitialized memory reads */
 	if (setjmp(env)) {
 		g_assert_not_reached(); /* We never longjmp() */
@@ -146,7 +150,10 @@ rand31_random_seed(void)
 	nsecs += now.tv_nsec;
 	entropy_delay();
 	tm_precise_time(&now);
-	seed = UINT32_ROTL(seed, popcount(nsecs + now.tv_nsec));
+	seed += integer_hash2(now.tv_sec + now.tv_nsec);
+	nsecs += now.tv_nsec;
+	nsecs %= 31;
+	seed = UINT32_ROTL(seed, nsecs);
 	while (0 != discard--) {
 		seed = rand31_prng_next(seed);
 	}
