@@ -48,17 +48,26 @@
 
 typedef void (*signal_handler_t)(int signo);
 
+/**
+ * Cleanup routines that can be installed with signal_cleanup_add().
+ */
+typedef void (*signal_cleanup_t)(void);
+
 /*
  * Public interface.
  */
 
 signal_handler_t signal_set(int signo, signal_handler_t handler);
 signal_handler_t signal_catch(int signo, signal_handler_t handler);
+void signal_cleanup_add(signal_cleanup_t cleanup);
+void signal_perform_cleanup(void);
 const char *signal_name(int signo);
 bool signal_in_handler(void);
 struct ckhunk *signal_chunk(void);
 void signal_unblock(int signo);
-void signal_abort(void);
+void signal_abort(void) G_GNUC_NORETURN;
+size_t signal_stack_allocate(void **base_ptr);
+bool signal_stack_free(void **base_ptr);
 
 bool signal_enter_critical(sigset_t *oset);
 void signal_leave_critical(const sigset_t *oset);
