@@ -39,6 +39,7 @@
 #include "atomic.h"
 #include "bfd_util.h"
 #include "concat.h"
+#include "constants.h"
 #include "crash.h"		/* For print_str() and crash_signame() */
 #include "dl_util.h"
 #include "eslist.h"
@@ -93,6 +94,7 @@ static symbols_t *symbols;
 static bool stacktrace_inited;
 
 static mutex_t stacktrace_atom_mtx = MUTEX_INIT;
+static mutex_t stacktrace_sym_mtx  = MUTEX_INIT;
 static once_flag_t stacktrace_atom_inited;
 
 #define STACKTRACE_ATOM_LOCK	mutex_lock(&stacktrace_atom_mtx)
@@ -101,27 +103,8 @@ static once_flag_t stacktrace_atom_inited;
 #define assert_stacktrace_atom_locked() \
 	assert_mutex_is_owned(&stacktrace_atom_mtx)
 
-/**
- * The buffers are allocated to construct the stack trace atomically to make
- * sure it can be logged as a whole.
- */
-struct stackbuf {
-	str_t *s;				/**< String buffer, allocated once */
-	slink_t lnk;			/**< List of buffers */
-};
-
-static struct {
-	spinlock_t lock;		/**< Thread-safe lock */
-	eslist_t buffers;		/**< List of allocated string buffers */
-	int init_count;			/**< Whether it was inited */
-} stacktrace_buffer = {
-	SPINLOCK_INIT,									/* lock */
-	ESLIST_INIT(offsetof(struct stackbuf, lnk)),	/* buffers */
-	0,												/* init_count */
-};
-
-#define STACKTRACE_BUFFER_LOCK		spinlock_hidden(&stacktrace_buffer.lock)
-#define STACKTRACE_BUFFER_UNLOCK	spinunlock_hidden(&stacktrace_buffer.lock)
+#define STACKTRACE_SYM_LOCK		mutex_lock(&stacktrace_sym_mtx)
+#define STACKTRACE_SYM_UNLOCK	mutex_unlock(&stacktrace_sym_mtx)
 
 /**
  * Auto-tuning stack trace offset.
@@ -222,12 +205,10 @@ stacktrace_gcc_unwind(void *stack[], size_t count, size_t offset)
 		return 0;
 
 	for (i = 0; i < offset; i++) {
-		void *nframe = getframeaddr(i + 1);
+		frame = getframeaddr(i + 1);
 
-		if (!valid_stack_ptr(nframe))
+		if (!valid_stack_ptr(frame))
 			return 0;
-
-		frame = nframe;
 	}
 
 	/*
@@ -235,15 +216,13 @@ stacktrace_gcc_unwind(void *stack[], size_t count, size_t offset)
 	 */
 
 	for (;; i++) {
-		void *nframe = getframeaddr(i + 1);
+		frame = getframeaddr(i + 1);
 
-		if (!valid_stack_ptr(nframe) || i - offset >= count)
+		if (!valid_stack_ptr(frame) || i - offset >= count)
 			break;
 
 		if (!valid_ptr(stack[i - offset] = getreturnaddr(i)))
 			break;
-
-		frame = nframe;
 	}
 
 	return i - offset;
@@ -542,92 +521,22 @@ stacktrace_auto_tune(void)
 static void G_GNUC_COLD
 stacktrace_get_symbols(const char *path, const char *lpath, bool stale)
 {
-	static spinlock_t sym_get_slk = SPINLOCK_INIT;
-
 	/*
 	 * In case we're crashing so early that stacktrace_init() has not been
 	 * called, initialize properly.
 	 */
 
-	spinlock(&sym_get_slk);
+	STACKTRACE_SYM_LOCK;
 
 	if (NULL == symbols)
 		symbols = symbols_make(STACKTRACE_DLFT_SYMBOLS, TRUE);
 
 	symbols_load_from(symbols, path, lpath != NULL ? lpath : path);
 
-	spinunlock(&sym_get_slk);
-
 	if (stale)
 		symbols_mark_stale(symbols);
-}
 
-/**
- * Initialize the buffer used to atomically construct a stack trace.
- */
-static G_GNUC_COLD void
-stacktrace_buffer_init(void)
-{
-	int i;
-
-	if G_LIKELY(0 != stacktrace_buffer.init_count)
-		return;		/* Already initialized */
-
-	if G_UNLIKELY(0 != atomic_int_inc(&stacktrace_buffer.init_count))
-		return;		/* Race condition, someone else is initializing */
-
-	if (spinlock_in_crash_mode())
-		return;		/* Too late, risking fatal race during eslist setup */
-
-	/*
-	 * Populate some stacktrace buffer strings.
-	 */
-
-	for (i = 0; i < STACKTRACE_BUFFER_COUNT; i++) {
-		struct stackbuf *sb = vmm_alloc_not_leaking(STACKTRACE_BUFFER_SIZE);
-
-		/*
-		 * The stackbuf structure is at the top of the buffer, followed by
-		 * the str_t object, followed by the string arena.
-		 */
-
-		sb->s = str_new_in_buffer(&sb[1],
-			STACKTRACE_BUFFER_SIZE - ptr_diff(&sb[1], sb));
-
-		STACKTRACE_BUFFER_LOCK;
-		eslist_append(&stacktrace_buffer.buffers, sb);
-		STACKTRACE_BUFFER_UNLOCK;
-	}
-}
-
-/**
- * Attempt to get a free stacktrace buffer string.
- *
- * @return a new stacktrace buffer string, NULL if none is available.
- */
-static struct stackbuf *
-stacktrace_buffer_get(void)
-{
-	struct stackbuf *sb;
-
-	stacktrace_buffer_init();
-
-	STACKTRACE_BUFFER_LOCK;
-	sb = eslist_shift(&stacktrace_buffer.buffers);
-	STACKTRACE_BUFFER_UNLOCK;
-
-	return sb;
-}
-
-/**
- * Put stacktrace buffer string back to the pool.
- */
-static void
-stacktrace_buffer_release(struct stackbuf *sb)
-{
-	STACKTRACE_BUFFER_LOCK;
-	eslist_append(&stacktrace_buffer.buffers, sb);
-	STACKTRACE_BUFFER_UNLOCK;
+	STACKTRACE_SYM_UNLOCK;
 }
 
 /**
@@ -654,7 +563,6 @@ stacktrace_init(const char *argv0, bool deferred)
 	path = program_path_allocate(argv0);
 	if (NULL == symbols)
 		symbols = symbols_make(STACKTRACE_DLFT_SYMBOLS, TRUE);
-	stacktrace_buffer_init();
 
 	if (NULL == path)
 		goto done;
@@ -866,6 +774,17 @@ stack_print(FILE *f, void * const *stack, size_t count)
 
 	stacktrace_load_symbols();
 
+	/*
+	 * Since symbols_name() returns pointer to static data, we need to protect
+	 * against concurrent calls.
+	 *
+	 * As a side effect, this prevents two stacks from two threads from being
+	 * inter-mixed in the output.
+	 *		--RAM, 2015-10-01
+	 */
+
+	STACKTRACE_SYM_LOCK;
+
 	for (i = 0; i < count; i++) {
 		const char *where = symbols_name(symbols, stack[i], TRUE);
 
@@ -876,6 +795,8 @@ stack_print(FILE *f, void * const *stack, size_t count)
 		if (stack_reached_main(where))
 			break;
 	}
+
+	STACKTRACE_SYM_UNLOCK;
 }
 
 /**
@@ -892,6 +813,17 @@ stack_log(logagent_t *la, void * const *stack, size_t count)
 
 	stacktrace_load_symbols();
 
+	/*
+	 * Since symbols_name() returns pointer to static data, we need to protect
+	 * against concurrent calls.
+	 *
+	 * As a side effect, this prevents two stacks from two threads from being
+	 * inter-mixed in the output.
+	 *		--RAM, 2015-10-01
+	 */
+
+	STACKTRACE_SYM_LOCK;
+
 	for (i = 0; i < count; i++) {
 		const char *where = symbols_name(symbols, stack[i], TRUE);
 
@@ -902,6 +834,8 @@ stack_log(logagent_t *la, void * const *stack, size_t count)
 		if (stack_reached_main(where))
 			break;
 	}
+
+	STACKTRACE_SYM_UNLOCK;
 }
 
 /**
@@ -916,6 +850,17 @@ stack_safe_print(int fd, void * const *stack, size_t count)
 {
 	size_t i;
 
+	/*
+	 * Since symbols_name() returns pointer to static data, we need to protect
+	 * against concurrent calls.
+	 *
+	 * As a side effect, this prevents two stacks from two threads from being
+	 * inter-mixed in the output.
+	 *		--RAM, 2015-10-01
+	 */
+
+	STACKTRACE_SYM_LOCK;
+
 	for (i = 0; i < count; i++) {
 		const char *where = symbols_name(symbols, stack[i], TRUE);
 		DECLARE_STR(3);
@@ -931,6 +876,8 @@ stack_safe_print(int fd, void * const *stack, size_t count)
 		if (stack_reached_main(where))
 			break;
 	}
+
+	STACKTRACE_SYM_UNLOCK;
 }
 
 /**
@@ -1058,12 +1005,29 @@ stack_print_decorated_to(struct sxfile *xf,
 	static char buf[512];
 	static char name[256];
 	static char tid[32];
-	str_t s, *trace = NULL;
-	struct stackbuf *sb = NULL;
+	str_t s;
 	bool gdb_like = booleanize(flags & STACKTRACE_F_GDB);
 	bool reached_main = FALSE;
 	int saved_errno = errno;
 
+	/*
+	 * We're using global variables, and we need to avoid concurrent updates
+	 * if we want to have something that makes sense in the output.
+	 *
+	 * If we are not crashing, locks are still enabled so we can create
+	 * a critical section here to avoid garbling output.
+	 *
+	 * If we are crashing, and other threads reach this point, they are
+	 * going to be suspended if not already done when they attempt to grab
+	 * the lock: only the crashing thread will get a lock pass-through.
+	 *
+	 * This critical section alone also ensures that we never mix the outputs
+	 * of two threads attempting to dump a stack at the same time.
+	 *		--RAM, 2015-10-01
+	 */
+
+	STACKTRACE_SYM_LOCK;
+
 	/*
 	 * The BFD environment is only opened once.
 	 * See rationale at the end of this routine.
@@ -1074,20 +1038,6 @@ stack_print_decorated_to(struct sxfile *xf,
 
 	str_new_buffer(&s, buf, 0, sizeof buf);
 
-	/*
-	 * If we can grab a pre-allocated buffer, use it to construct the stack
-	 * trace so that we can atomically emit it in the logs without possible
-	 * output from other threads being intermixed.
-	 */
-
-	sb = stacktrace_buffer_get();
-
-	if (NULL != sb) {
-		trace = sb->s;
-		g_assert(trace != NULL);
-		str_reset(trace);
-	}
-
 	/*
 	 * Compute leading thread ID, shown only when not in the main thread.
 	 */
@@ -1317,36 +1267,14 @@ stack_print_decorated_to(struct sxfile *xf,
 
 		str_putc(&s, '\n');
 
-		if (NULL == trace) {
-			switch (xf->type) {
-			case SXFILE_STDIO:
-				atio_fwrite(str_2c(&s), str_len(&s), 1, xf->u.f);
-				break;
-			case SXFILE_FD:
-				atio_write(xf->u.fd, str_2c(&s), str_len(&s));
-				break;
-			}
-		} else {
-			/* This will silently truncate output if buffer is too small */
-			str_ncat_safe(trace, str_2c(&s), str_len(&s));
-		}
-	}
-
-	/*
-	 * Emit the constructed trace if we were holding output.
-	 */
-
-	if (trace != NULL) {
 		switch (xf->type) {
 		case SXFILE_STDIO:
-			atio_fwrite(str_2c(trace), str_len(trace), 1, xf->u.f);
+			atio_fwrite(str_2c(&s), str_len(&s), 1, xf->u.f);
 			break;
 		case SXFILE_FD:
-			atio_write(xf->u.fd, str_2c(trace), str_len(trace));
+			atio_write(xf->u.fd, str_2c(&s), str_len(&s));
 			break;
 		}
-		g_assert(sb != NULL);
-		stacktrace_buffer_release(sb);
 	}
 
 	/*
@@ -1369,6 +1297,8 @@ stack_print_decorated_to(struct sxfile *xf,
 	 */
 
 	errno = saved_errno;
+
+	STACKTRACE_SYM_UNLOCK;
 }
 
 /**
@@ -1479,6 +1409,8 @@ stacktrace_caller_name(size_t n)
 {
 	void *stack[STACKTRACE_DEPTH_MAX];
 	size_t count;
+	bool in_sigh = signal_in_handler();
+	const char *name;
 
 	g_assert(size_is_non_negative(n));
 	g_assert(n <= STACKTRACE_DEPTH_MAX);
@@ -1487,10 +1419,27 @@ stacktrace_caller_name(size_t n)
 	if (n >= count)
 		return "";
 
-	if (!signal_in_handler())
+	if (!in_sigh)
 		stacktrace_load_symbols();
 
-	return NULL == symbols ? "??" : symbols_name(symbols, stack[n], FALSE);
+	if (NULL == symbols)
+		return "??";
+
+	STACKTRACE_SYM_LOCK;
+
+	name = symbols_name(symbols, stack[n], FALSE);
+
+	/*
+	 * Avoid all memory allocation if we are in a signal handler or
+	 * crashing (where normally we are supposed to be running mono-threaded).
+	 */
+
+	if (!in_sigh && !thread_in_crash_mode())
+		name = constant_str(name);
+
+	STACKTRACE_SYM_UNLOCK;
+
+	return name;
 }
 
 /**
@@ -1507,10 +1456,13 @@ const char *
 stacktrace_routine_name(const void *pc, bool offset)
 {
 	const char *name;
+	bool in_sigh = signal_in_handler();
 
-	if (!signal_in_handler())
+	if (!in_sigh)
 		stacktrace_load_symbols();
 
+	STACKTRACE_SYM_LOCK;
+
 	name = NULL == symbols ? NULL : symbols_name_only(symbols, pc, offset);
 
 	/*
@@ -1529,9 +1481,11 @@ stacktrace_routine_name(const void *pc, bool offset)
 	if (NULL == name) {
 		static char buf[POINTER_BUFLEN + CONST_STRLEN("0x")];
 		str_bprintf(buf, sizeof buf, "%p", pc);
-		name = buf;
+		name = (in_sigh || thread_in_crash_mode()) ? buf : constant_str(buf);
 	}
 
+	STACKTRACE_SYM_UNLOCK;
+
 	return name;
 }
 
