@@ -33,16 +33,19 @@
 #include "common.h"
 
 #include "aq.h"
+#include "atio.h"
 #include "atomic.h"
 #include "barrier.h"
 #include "compat_poll.h"
 #include "compat_sleep_ms.h"
 #include "cond.h"
 #include "cq.h"
+#include "crash.h"
 #include "dam.h"
 #include "evq.h"
 #include "getcpucount.h"
 #include "halloc.h"
+#include "hset.h"
 #include "log.h"
 #include "misc.h"
 #include "mutex.h"
@@ -58,6 +61,7 @@
 #include "stacktrace.h"
 #include "str.h"
 #include "stringify.h"
+#include "strtok.h"
 #include "teq.h"
 #include "thread.h"
 #include "tm.h"
@@ -87,7 +91,8 @@ usage(void)
 {
 	fprintf(stderr,
 		"Usage: %s [-hejsvwxABCDEFIKMNOPQRSVWX] [-a type] [-b size] [-c CPU]\n"
-		"       [-f count] [-n count] [-r percent] [-t ms] [-T secs]\n"
+		"       [-f count] [-n count] [-r percent] [-t ms] [-T msecs]\n"
+		"       [-z fn1,fn2...]\n"
 		"  -a : allocator to exlusively test via -X (see below for type)\n"
 		"  -b : fixed block size to use for memory tests via -X\n"
 		"  -c : override amount of CPUs, driving thread count for mem tests\n"
@@ -102,6 +107,7 @@ usage(void)
 		"  -v : dump thread statistics at the end of the tests\n"
 		"  -w : wait for created threads\n"
 		"  -x : free memory allocated by -X in random order\n"
+		"  -z : zap (suppress) messages from listed routines.\n"
 		"  -A : use asynchronous exit callbacks\n"
 		"  -B : test synchronization barriers\n"
 		"  -C : test thread creation\n"
@@ -117,7 +123,7 @@ usage(void)
 		"  -Q : test asynchronous queue\n"
 		"  -R : test the read-write lock layer\n"
 		"  -S : test semaphore layer\n"
-		"  -T : test condition layer via tennis session for specified secs\n"
+		"  -T : test condition layer via tennis session for specified msecs\n"
 		"  -V : test thread event queue (TEQ)\n"
 		"  -W : test local event queue (EVQ)\n"
 		"  -X : exercise concurrent memory allocation\n"
@@ -127,6 +133,73 @@ usage(void)
 	exit(EXIT_FAILURE);
 }
 
+static hset_t *zap;
+
+static void
+zap_record(const char *value)
+{
+	strtok_t *s;
+	const char *tok;
+
+	zap = hset_create(HASH_KEY_STRING, 0);
+	s = strtok_make_strip(value);
+
+	while ((tok = strtok_next(s, ","))) {
+		hset_insert(zap, h_strdup(tok));
+	}
+
+	strtok_free_null(&s);
+}
+
+static void
+emitv(bool nl, const char *fmt, va_list args)
+{
+	str_t *s = str_new(512);
+	iovec_t iov[2];
+	int cnt = 0;
+
+	str_vprintf(s, fmt, args);
+	iovec_set(&iov[cnt++], str_2c(s), str_len(s));
+	if (nl)
+		iovec_set(&iov[cnt++], "\n", 1);
+
+	/*
+	 * Emit every message to stderr, since this is the same channel used
+	 * by s_debug() and friends.  That way, all the output goes to the
+	 * very same descriptor, atomically, avoiding garbled output.
+	 */
+
+	atio_writev(STDERR_FILENO, iov, cnt);
+
+	str_destroy_null(&s);
+}
+
+static void G_GNUC_PRINTF(1, 2)
+emit(const char *fmt, ...)
+{
+	va_list args;
+
+	va_start(args, fmt);
+	emitv(TRUE, fmt, args);
+	va_end(args);
+}
+
+static void G_GNUC_PRINTF(2, 3)
+emit_zap(const char *caller, const char *fmt, ...)
+{
+	va_list args;
+
+	if (zap != NULL && hset_contains(zap, caller))
+		return;		/* Zap messages from this caller */
+
+	va_start(args, fmt);
+	emitv(TRUE, fmt, args);
+	va_end(args);
+}
+
+#define emitz(fmt, ...) emit_zap(G_STRFUNC, (fmt), __VA_ARGS__)
+#define TESTING(func)	emit("----------- %s() -----------", (func));
+
 static char *names[] = { "one", "two", "three", "four", "five" };
 
 static void
@@ -134,8 +207,7 @@ exit_callback(void *result, void *arg)
 {
 	char *name = arg;
 	long length = pointer_to_long(result);
-	printf("thread \"%s\" finished, result length is %ld\n", name, length);
-	fflush(stdout);
+	emit("thread \"%s\" finished, result length is %ld", name, length);
 }
 
 static void *
@@ -145,8 +217,7 @@ compute_length(void *arg)
 	const char *name = arg;
 	char *scratch = xstrdup(name);
 
-	printf("%s given \"%s\"\n", thread_id_name(stid), scratch);
-	fflush(stdout);
+	emit("%s given \"%s\"", thread_id_name(stid), scratch);
 	xfree(scratch);
 	thread_cancel_test();
 	if (sleep_before_exit)
@@ -181,18 +252,18 @@ test_create_one(bool repeat, bool join)
 		} else {
 			int j;
 			if (!repeat)
-				printf("thread i=%u created as %s\n", i, thread_id_name(r));
+				emit("thread i=%u created as %s", i, thread_id_name(r));
 			if (!join) {
 				j = thread_join(r, NULL);
 				if (-1 != j) {
-					s_warning("thread_join() worked for %s?\n",
+					s_warning("thread_join() worked for %s?",
 						thread_id_name(r));
 				} else if (errno != EINVAL) {
 					s_warning("thread_join() failure on %s: %m",
 						thread_id_name(r));
 				} else {
 					if (!repeat) {
-						printf("%s cannot be joined, that's OK\n",
+						emit("%s cannot be joined, that's OK",
 							thread_id_name(r));
 					}
 					if (wait_threads) {
@@ -223,13 +294,13 @@ test_create_one(bool repeat, bool join)
 		compat_sleep_ms(200);		/* Let all the threads run */
 
 	if (join) {
-		printf("now joining the %u threads\n", (uint) G_N_ELEMENTS(launched));
+		emit("now joining the %u threads", (uint) G_N_ELEMENTS(launched));
 		for (i = 0; i < G_N_ELEMENTS(launched); i++) {
 			int r = launched[i];
 
 			if (-1 == r) {
 				if (!repeat)
-					printf("skipping unlaunched thread i=%u\n", i);
+					emit("skipping unlaunched thread i=%u", i);
 			} else {
 				void *result;
 				int j;
@@ -246,7 +317,7 @@ test_create_one(bool repeat, bool join)
 				} else {
 					ulong length = pointer_to_ulong(result);
 					if (!repeat) {
-						printf("%s finished, result length is %lu\n",
+						emit("%s finished, result length is %lu",
 							thread_id_name(r), length);
 					}
 				}
@@ -292,9 +363,8 @@ posix_worker(void *unused_arg)
 	thread_current_info(&info);
 	thread_info_to_string_buf(&info, name, sizeof name);
 
-	printf("POSIX thread worker starting...\n");
-	printf("POSIX worker: %s\n", name);
-	fflush(stdout);
+	emit("POSIX thread worker starting...");
+	emit("POSIX worker: %s", name);
 
 	for (;;) {
 		void *p;
@@ -326,26 +396,46 @@ posix_threads(void *unused_arg)
 
 	(void) unused_arg;
 
-	printf("POSIX thread launcher starting...\n");
-	fflush(stdout);
+	emit("POSIX thread launcher starting...");
 
 	for (i = 0; i < 6; i++) {
 		(void) posix_thread_create(posix_worker, NULL, FALSE);
 	}
 
-	printf("POSIX thread launch done, mutating to worker...\n");
-	fflush(stdout);
+	emit("POSIX thread launch done, mutating to worker...");
 
 	return posix_worker(NULL);
 }
 
+static void *
+posix_exiting_thread(void *unused_arg)
+{
+	void *p;
+	(void) unused_arg;
+
+	p = xmalloc(100);
+	compat_sleep_ms(100);
+	xfree(p);
+
+	return NULL;
+}
+
 static void
 test_create(unsigned repeat, bool join, bool posix)
 {
 	unsigned i;
 
+	TESTING(G_STRFUNC);
+
 	if (posix) {
+		pthread_t foreign;
+
+		foreign = posix_thread_create(posix_exiting_thread, NULL, TRUE);
 		(void) posix_thread_create(posix_threads, NULL, FALSE);
+
+		emit("Waiting for single foreign POSIX thread...");
+		if (0 != pthread_join(foreign, NULL))
+			s_error("cannot wait for foreign POSIX thread");
 	}
 
 	for (i = 0; i < repeat; i++) {
@@ -380,20 +470,20 @@ test_cancel_one(bool repeat, bool join)
 		} else {
 			int j;
 			if (!repeat)
-				printf("thread i=%u created as %s\n", i, thread_id_name(r));
+				emit("thread i=%u created as %s", i, thread_id_name(r));
 			if (-1 == thread_cancel(r))
 				s_warning("thread_cancel(%u) failed: %m", r);
 			if (!join) {
 				j = thread_join(r, NULL);
 				if (-1 != j) {
-					s_warning("thread_join() worked for %s?\n",
+					s_warning("thread_join() worked for %s?",
 						thread_id_name(r));
 				} else if (errno != EINVAL) {
 					s_warning("thread_join() failure on %s: %m",
 						thread_id_name(r));
 				} else {
 					if (!repeat) {
-						printf("%s cannot be joined, that's OK\n",
+						emit("%s cannot be joined, that's OK",
 							thread_id_name(r));
 					}
 				}
@@ -405,13 +495,13 @@ test_cancel_one(bool repeat, bool join)
 		compat_sleep_ms(200);		/* Let all the threads run */
 
 	if (join) {
-		printf("now joining the %u threads\n", (uint) G_N_ELEMENTS(launched));
+		emit("now joining the %u threads", (uint) G_N_ELEMENTS(launched));
 		for (i = 0; i < G_N_ELEMENTS(launched); i++) {
 			int r = launched[i];
 
 			if (-1 == r) {
 				if (!repeat)
-					printf("skipping unlaunched thread i=%u\n", i);
+					emit("skipping unlaunched thread i=%u", i);
 			} else {
 				void *result;
 				int j = thread_join(r, &result);		/* Block */
@@ -421,7 +511,7 @@ test_cancel_one(bool repeat, bool join)
 				} else {
 					long length = pointer_to_long(result);
 					if (!repeat) {
-						printf("%s finished, result length is %ld\n",
+						emit("%s finished, result length is %ld",
 							thread_id_name(r), length);
 					}
 				}
@@ -432,9 +522,7 @@ test_cancel_one(bool repeat, bool join)
 	if (async_exit)
 		cq_main_dispatch();
 
-	i = thread_create(sleeping_thread, NULL, 0, STACK_SIZE);
-	if (-1U == i)
-		s_error("cannot create sleeping thread: %m");
+	i = thread_create(sleeping_thread, NULL, THREAD_F_PANIC, STACK_SIZE);
 	if (-1 == thread_cancel(i)) {
 		s_error("cannot cancel sleeping thread: %m");
 	} else {
@@ -451,6 +539,8 @@ test_cancel(unsigned repeat, bool join)
 {
 	unsigned i;
 
+	TESTING(G_STRFUNC);
+
 	for (i = 0; i < repeat; i++) {
 		test_cancel_one(repeat > 1, join);
 	}
@@ -462,8 +552,7 @@ test_inter_main(void *arg)
 	waiter_t *w = waiter_refcnt_inc(arg);
 
 	sleep(1);
-	printf("signaling main thread\n");
-	fflush(stdout);
+	emit("signaling main thread");
 	waiter_signal(w);
 	
 	compat_sleep_ms(5);
@@ -474,32 +563,29 @@ test_inter_main(void *arg)
 static void
 test_inter(void)
 {
-	int r;
 	waiter_t *mw, *w;
 	bool refed;
 
+	TESTING(G_STRFUNC);
+
 	mw = waiter_make(NULL);
 	w = waiter_spawn(mw, int_to_pointer(31416));
-	
-	r = thread_create(test_inter_main, w, THREAD_F_DETACH, 0);
-	if (-1 == r)
-		s_error("could not launch thread: %m");
 
-	printf("main thread waiting\n");
-	fflush(stdout);
+	thread_create(test_inter_main, w, THREAD_F_DETACH | THREAD_F_PANIC, 0);
+
+	emit("main thread waiting");
 	if (!waiter_suspend(mw))
 		s_error("could not suspend itself");
-	printf("main thread awoken\n");
+	emit("main thread awoken");
 
 	refed = waiter_refcnt_dec(w);
-	printf("child waiter %s referenced\n", refed ? "still" : "no longer");
+	emit("child waiter %s referenced", refed ? "still" : "no longer");
 	while (waiter_child_count(mw) != 1) {
-		printf("waiting for all children in waiter to go\n");
-		fflush(stdout);
+		emit("waiting for all children in waiter to go");
 		compat_sleep_ms(5);
 	}
 	refed = waiter_refcnt_dec(mw);
-	printf("master waiter %s referenced\n", refed ? "still" : "no longer");
+	emit("master waiter %s referenced", refed ? "still" : "no longer");
 }
 
 struct test_semaphore_arg {
@@ -519,24 +605,21 @@ test_semaphore_main(void *arg)
 
 	for (i = 0; i < 6; i++) {
 		bool got = TRUE;
-		printf("thread #%d alive, waiting for event\n", n);
-		fflush(stdout);
+		emit("thread #%d alive, waiting for event", n);
 		if (semaphore_acquire(s, 1, NULL)) {
-			printf("#%d got semaphore!\n", n);
-			fflush(stdout);
+			emit("#%d got semaphore!", n);
 		} else {
 			s_warning("thread #%d cannot get semaphore: %m", n);
 			got = FALSE;
 		}
 		sleep(1);
 		if (got) {
-			printf("#%d releasing semaphore!\n", n);
-			fflush(stdout);
+			emit("#%d releasing semaphore!", n);
 			semaphore_release(s, 1);
 		}
 	}
 
-	printf("thread #%d exiting!\n", n);
+	emit("thread #%d exiting!", n);
 	return NULL;
 }
 
@@ -544,16 +627,12 @@ static int
 test_semaphore_thread_launch(int n, semaphore_t *s)
 {
 	struct test_semaphore_arg *arg;
-	int r;
 
 	XMALLOC(arg);
 	arg->n = n;
 	arg->s = s;
-	r = thread_create(test_semaphore_main, arg, 0, 0);
-	if (-1 == r)
-		s_error("could not launch thread #%d: %m", n);
 
-	return r;
+	return thread_create(test_semaphore_main, arg, THREAD_F_PANIC, 0);
 }
 
 static void
@@ -564,6 +643,8 @@ test_semaphore(bool emulated)
 	int i;
 	int r[3];
 
+	TESTING(G_STRFUNC);
+
 	s = semaphore_create_full(0, emulated);
 	if (semaphore_acquire_try(s, 1))
 		s_error("could acquire empty semaphore!");
@@ -577,12 +658,12 @@ test_semaphore(bool emulated)
 		s_error("could not acquire semaphore, second time!");
 
 	tm_fill_ms(&timeout, 500);
-	printf("will wait 1/2 second...\n");
+	emit("will wait 1/2 second...");
 	if (semaphore_acquire(s, 1, &timeout))
 		s_error("could acquire empty semaphore!");
 	g_assert_log(EAGAIN == errno,
 		"improper errno, expected EAGAIN (%d) but got %m", EAGAIN);
-	printf("good, failed to acquire empty semaphore.\n");
+	emit("good, failed to acquire empty semaphore.");
 
 	r[0] = test_semaphore_thread_launch(1, s);
 	r[1] = test_semaphore_thread_launch(2, s);
@@ -591,20 +672,18 @@ test_semaphore(bool emulated)
 	semaphore_release(s, 1);
 
 	for (i = 0; i < 6; i++) {
-		printf("main alive\n");
-		fflush(stdout);
+		emit("main alive");
 		sleep(1);
 	}
 
-	printf("main waiting for subthreads\n");
-	fflush(stdout);
+	emit("main waiting for subthreads");
 
 	for (i = 0; UNSIGNED(i) < G_N_ELEMENTS(r); i++) {
 		if (-1 == thread_join(r[i], NULL))
 			s_error("failed to join with %s: %m", thread_id_name(r[i]));
 	}
 
-	printf("main is done, final semaphore value is %d\n", semaphore_value(s));
+	emit("main is done, final semaphore value is %d", semaphore_value(s));
 
 	semaphore_destroy(&s);
 }
@@ -671,8 +750,7 @@ player(void *num)
 
 	while (game_state < GAME_OVER) {
 		stats->play++;
-		printf("%s plays\n", me);
-		fflush(stdout);
+		emitz("%s plays", me);
 
 		game_state = other_player;
 		cond_signal(&game_state_change, &game_state_lock);
@@ -686,8 +764,7 @@ player(void *num)
 
 				if (!success) {
 					stats->timeout++;
-					printf("** TIMEOUT ** %s wakeup\n", me);
-					fflush(stdout);
+					emitz("** TIMEOUT ** %s wakeup", me);
 					continue;
 				}
 			} else {
@@ -696,15 +773,13 @@ player(void *num)
 
 			if (other_player == game_state) {
 				stats->spurious++;
-				printf("** SPURIOUS ** %s wakeup\n", me);
-				fflush(stdout);
+				emitz("** SPURIOUS ** %s wakeup", me);
 			}
 		} while (other_player == game_state);
 	}
 
 	game_state++;
-	printf("%s leaving\n", me);
-	fflush(stdout);
+	emit("%s leaving", me);
 
 	cond_broadcast(&game_state_change, &game_state_lock);
 	mutex_unlock(&game_state_lock);
@@ -721,7 +796,7 @@ player_stats(int n)
 
 	stats = &game_stats[n];
 
-	printf("%s played %d times (%d spurious event%s, %d timeout%s)\n",
+	emit("%s played %d times (%d spurious event%s, %d timeout%s)",
 		name[n], stats->play, stats->spurious, plural(stats->spurious),
 		stats->timeout, plural(stats->timeout));
 }
@@ -734,6 +809,8 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 	waiter_t *w;
 	uint notifications = 0;
 
+	TESTING(G_STRFUNC);
+
 	g_assert(0 == cond_waiting_count(&game_state_change));
 
 	if (emulated)
@@ -752,7 +829,7 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 		tm_t end, play;
 
 		tm_now_exact(&end);
-		tm_fill_ms(&play, play_time * 1000);
+		tm_fill_ms(&play, play_time);
 		tm_add(&end, &play);
 
 		w = waiter_make(NULL);
@@ -783,8 +860,7 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 			if (ret < 0) {
 				s_warning("poll() failed: %m");
 			} else {
-				printf("[match still on]\n");
-				fflush(stdout);
+				emit_zap("player", "[match still on]");
 				notifications++;
 				waiter_ack(w);
 			}
@@ -794,24 +870,23 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 		g_assert(1 == waiter_refcnt(w));
 		waiter_refcnt_dec(w);
 	} else {
-		sleep(play_time);	/* Let them play */
+		compat_sleep_ms(play_time);	/* Let them play */
 	}
 
 	if (noise) {
-		printf("** Noise ON **\n");
+		emit("** Noise ON **");
 		for (i = 0; i < 100000; i++) {
 			mutex_lock(&game_state_lock);
 			cond_broadcast(&game_state_change, &game_state_lock);
 			mutex_unlock(&game_state_lock);
 		}
-		printf("** Noise OFF **\n");
+		emit("** Noise OFF **");
 	}
 
 	mutex_lock(&game_state_lock);
 	game_state = GAME_OVER;
 
-	printf("Stopping the game...\n");
-	fflush(stdout);
+	emit("Stopping the game...");
 
 	cond_broadcast(&game_state_change, &game_state_lock);
 
@@ -821,7 +896,7 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 
 	g_assert(0 == cond_waiting_count(&game_state_change));
 
-	printf("Game over!\n");
+	emit("Game over!");
 
 	mutex_unlock(&game_state_lock);
 	mutex_destroy(&game_state_lock);
@@ -838,7 +913,7 @@ test_condition(unsigned play_time, bool emulated, bool monitor, bool noise)
 		player_stats(i);
 	}
 	if (monitor) {
-		printf("main got %u notification%s\n",
+		emit("main got %u notification%s",
 			notifications, plural(notifications));
 	}
 }
@@ -856,21 +931,18 @@ fork_locker(void *arg)
 
 	thread_set_name(fork_names[n]);
 
-	printf("%s locking...\n", thread_name());
-	fflush(stdout);
+	emit("%s locking...", thread_name());
 
 	spinlock(&locks[n]);
 	mutex_lock(&mutexes[n]);
 	rwlock_rlock(&rlocks[n]);
 	rwlock_wlock(&wlocks[n]);
 
-	printf("%s locked, sleeping for 5 secs\n", thread_name());
-	fflush(stdout);
+	emit("%s locked, sleeping for 5 secs", thread_name());
 
 	sleep(5);
 
-	printf("%s unlocking\n", thread_name());
-	fflush(stdout);
+	emit("%s unlocking", thread_name());
 
 	rwlock_wunlock(&wlocks[n]);
 	rwlock_runlock(&rlocks[n]);
@@ -887,14 +959,12 @@ fork_forker(void *arg)
 	unsigned running;
 	bool safe = booleanize(pointer_to_int(arg));
 
-	printf("%s() waiting 1 sec\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s() waiting 1 sec", G_STRFUNC);
 	sleep(1);
 
 	running = thread_count();
-	printf("%s() forking with %u running thread%s, STID=%u\n", G_STRFUNC,
+	emit("%s() forking with %u running thread%s, STID=%u", G_STRFUNC,
 		running, plural(running), thread_small_id());
-	fflush(stdout);
 
 	thread_lock_dump_all(STDOUT_FILENO);
 
@@ -902,15 +972,13 @@ fork_forker(void *arg)
 	case -1:
 		s_error("%s() cannot fork(): %m", G_STRFUNC);
 	case 0:
-		printf("%s() child process started as STID=%u\n",
+		emit("%s() child process started as STID=%u",
 			G_STRFUNC, thread_small_id());
-		printf("%s() child has %u thread\n", G_STRFUNC, thread_count());
-		fflush(stdout);
+		emit("%s() child has %u thread", G_STRFUNC, thread_count());
 		thread_lock_dump_all(STDOUT_FILENO);
 		exit(EXIT_SUCCESS);
 	default:
-		printf("%s() child forked, waiting...\n", G_STRFUNC);
-		fflush(stdout);
+		emit("%s() child forked, waiting...", G_STRFUNC);
 #ifdef HAS_WAITPID
 		{
 			pid_t w = waitpid(pid, NULL, 0);
@@ -918,8 +986,7 @@ fork_forker(void *arg)
 				s_error("%s() cannot wait(): %m", G_STRFUNC);
 		}
 #endif
-		printf("%s() child terminated, exiting thread\n", G_STRFUNC);
-		fflush(stdout);
+		emit("%s() child terminated, exiting thread", G_STRFUNC);
 	}
 
 	return NULL;
@@ -931,18 +998,16 @@ test_fork(bool safe)
 	int l1, l2, fk, r;
 	unsigned running;
 
-	printf("--- testing thread_fork(%s)\n", safe ? "TRUE" : "FALSE");
+	TESTING(G_STRFUNC);
+
+	emit("--- testing thread_fork(%s)", safe ? "TRUE" : "FALSE");
 
 	running = thread_count();
-	printf("starting with %u running thread%s\n",
-		running, plural(running));
+	emit("starting with %u running thread%s", running, plural(running));
 
-	l1 = thread_create(fork_locker, int_to_pointer(0), 0, 8192);
-	l2 = thread_create(fork_locker, int_to_pointer(1), 0, 8192);
-	fk = thread_create(fork_forker, int_to_pointer(safe), 0, 8192);
-
-	if (-1 == l1 || -1 == l2 || -1 == fk)
-		s_error("%s() could not create threads", G_STRFUNC);
+	l1 = thread_create(fork_locker, int_to_pointer(0), THREAD_F_PANIC, 8192);
+	l2 = thread_create(fork_locker, int_to_pointer(1), THREAD_F_PANIC, 8192);
+	fk = thread_create(fork_forker, int_to_pointer(safe), THREAD_F_PANIC, 8192);
 
 	r = thread_join(l1, NULL);
 	if (-1 == r)
@@ -955,10 +1020,9 @@ test_fork(bool safe)
 		s_error("final thread_join() failed: %m");
 
 	running = thread_count();
-	printf("ending with %u running thread%s\n",
-		running, plural(running));
+	emit("ending with %u running thread%s", running, plural(running));
 
-	printf("--- test of thread_fork(%s) done!\n", safe ? "TRUE" : "FALSE");
+	emit("--- test of thread_fork(%s) done!", safe ? "TRUE" : "FALSE");
 }
 
 static int
@@ -977,9 +1041,19 @@ overflow_routine(void *arg)
 	return overflow_routine(c) + c[1];
 }
 
+static void
+overflow_handler(int unused_sig)
+{
+	(void) unused_sig;
+
+	s_rawdebug("stack overflow signal properly caught!");
+}
+
 static void *
 overflow_thread(void *arg)
 {
+	thread_signal(TSIG_OVFLOW, overflow_handler);
+
 	return int_to_pointer(overflow_routine(arg));
 }
 
@@ -988,9 +1062,9 @@ test_overflow(void)
 {
 	int t, r;
 
-	t = thread_create(overflow_thread, int_to_pointer(0), 0, 8192);
-	if (-1 == t)
-		s_error("%s() could not create thread", G_STRFUNC);
+	TESTING(G_STRFUNC);
+
+	t = thread_create(overflow_thread, int_to_pointer(0), THREAD_F_PANIC, 8192);
 	r = thread_join(t, NULL);
 	if (-1 == r)
 		s_error("%s(): thread_join() failed: %m", G_STRFUNC);
@@ -1037,30 +1111,27 @@ test_aqueue(bool emulated)
 	int t;
 	uint i;
 
-	printf("%s() starting...\n", G_STRFUNC);
+	TESTING(G_STRFUNC);
+
+	emit("%s() starting...", G_STRFUNC);
 
 	arg.r = r = aq_make_full(emulated);		/* requests */
 	arg.a = a = aq_make_full(emulated);		/* answers */
 
-	t = thread_create(aqt_processor, &arg, 0, 0);
-	if (-1 == t)
-		s_error("cannot create processor thread: %m");
+	t = thread_create(aqt_processor, &arg, THREAD_F_PANIC, 0);
 
 	for (i = 0; i < G_N_ELEMENTS(names); i++) {
 		ulong res;
 
-		printf("computing length of \"%s\"\n", names[i]);
-		fflush(stdout);
+		emit("computing length of \"%s\"", names[i]);
 
 		aq_put(r, names[i]);
 		res = pointer_to_ulong(aq_remove(a));
 
-		printf("\t=> %lu bytes\n", res);
-		fflush(stdout);
+		emit("\t=> %lu bytes", res);
 	}
 
-	printf("sending EOF\n");
-	fflush(stdout);
+	emit("sending EOF");
 
 	aq_put(r, NULL);		/* Signals end to thread */
 	if (-1 == thread_join(t, NULL))
@@ -1069,7 +1140,7 @@ test_aqueue(bool emulated)
 	aq_refcnt_dec(a);
 	aq_refcnt_dec(r);
 
-	printf("%s() all done.\n", G_STRFUNC);
+	emit("%s() all done.", G_STRFUNC);
 }
 
 static rwlock_t rwsync = RWLOCK_INIT;
@@ -1121,13 +1192,14 @@ test_rwlock(void)
 	rwlock_t rw = RWLOCK_INIT;
 	unsigned i;
 
+	TESTING(G_STRFUNC);
+
 	s_info("%s starting, will be launching %s()", thread_name(),
 		stacktrace_function_name(test_rwthreads));
 
 	/* The mono-threaded "cannot fail" sequence */
 
-	printf("%s(): mono-threaded tests...\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s(): mono-threaded tests...", G_STRFUNC);
 
 	if (!rwlock_rlock_try(&rw))
 		s_error("cannot read-lock");
@@ -1149,13 +1221,11 @@ test_rwlock(void)
 	if (!rwlock_is_free(&rw))
 		s_error("lock should be free");
 
-	printf("%s(): mono-threaded tests succeded.\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s(): mono-threaded tests succeded.", G_STRFUNC);
 
 	/* Now for multi-threaded tests.... */
 
-	printf("%s(): multi-threaded tests...\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s(): multi-threaded tests...", G_STRFUNC);
 
 	for (i = 0; i < G_N_ELEMENTS(t); i++) {
 		t[i] = thread_create(test_rwthreads, NULL, 0, 0);
@@ -1167,8 +1237,7 @@ test_rwlock(void)
 		thread_join(t[i], NULL);
 	}
 
-	printf("%s(): multi-threaded tests done.\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s(): multi-threaded tests done.", G_STRFUNC);
 }
 
 static bool test_signals_done;
@@ -1179,15 +1248,13 @@ static int test_signals_count;
 static void
 test_sighandler(int sig)
 {
-	printf("%s received signal #%d\n", thread_name(), sig);
-	fflush(stdout);
+	emit("%s received signal #%d", thread_name(), sig);
 }
 
 static void
 test_sigdone(int sig)
 {
-	printf("%s got signal #%d, will exit\n", thread_name(), sig);
-	fflush(stdout);
+	emit("%s got signal #%d, will exit", thread_name(), sig);
 
 	test_signals_done = TRUE;
 }
@@ -1195,16 +1262,14 @@ test_sigdone(int sig)
 static void
 test_sigcount(int sig)
 {
-	printf("%s got signal #%d (count = %u)\n", thread_name(), sig,
+	emit("%s got signal #%d (count = %u)", thread_name(), sig,
 		++test_signals_count);
-	fflush(stdout);
 }
 
 static void
 test_printsig(int sig)
 {
-	printf("%s got signal #%d\n", thread_name(), sig);
-	fflush(stdout);
+	emit("%s got signal #%d", thread_name(), sig);
 }
 
 static void *
@@ -1228,14 +1293,12 @@ signalled_thread(void *unused_arg)
 		if (!thread_pause())
 			s_error("thread was not unblocked by signal");
 		if (3 == count++) {
-			printf("%s() will now get signal #4\n", G_STRFUNC);
-			fflush(stdout);
+			emit("%s() will now get signal #4", G_STRFUNC);
 			thread_sigmask(TSIG_SETMASK, &oset, NULL);
 		}
 	}
 
-	printf("%s() exiting\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s() exiting", G_STRFUNC);
 
 	return NULL;
 }
@@ -1247,13 +1310,24 @@ sleeping_thread(void *arg)
 	tm_t start, end;
 
 	thread_signal(TSIG_1, test_sigcount);
+	if (b != NULL)
+		barrier_wait(b);
+
 	tm_now_exact(&start);
 	thread_sleep_ms(2000);
 	tm_now_exact(&end);
 
-	printf("%s() slept %u ms (expected 2000 ms)\n", G_STRFUNC,
+	emit("%s() slept %u ms (expected 2000 ms)", G_STRFUNC,
 		(uint) tm_elapsed_ms(&end, &start));
-	fflush(stdout);
+
+	while (test_signals_count < TEST_SIGNALS_COUNT) {
+		emit("%s() got %d/%d signals so far", G_STRFUNC,
+			test_signals_count, TEST_SIGNALS_COUNT);
+		thread_pause();
+	}
+
+	if (b != NULL)
+		barrier_wait(b);
 
 	g_assert(TEST_SIGNALS_COUNT == test_signals_count);
 
@@ -1272,9 +1346,8 @@ sleeping_thread(void *arg)
 		tm_now_exact(&start);
 		if (thread_timed_sigsuspend(&oset, &timeout)) {
 			tm_now_exact(&end);
-			printf("%s() suspended %u ms before getting signal\n", G_STRFUNC,
+			emit("%s() suspended %u ms before getting signal", G_STRFUNC,
 				(uint) tm_elapsed_ms(&end, &start));
-			fflush(stdout);
 		} else {
 			g_assert_not_reached();
 		}
@@ -1285,9 +1358,8 @@ sleeping_thread(void *arg)
 			g_assert_not_reached();
 		} else {
 			tm_now_exact(&end);
-			printf("%s() suspended %u ms without getting signals\n", G_STRFUNC,
+			emit("%s() suspended %u ms without getting signals", G_STRFUNC,
 				(uint) tm_elapsed_ms(&end, &start));
-			fflush(stdout);
 		}
 
 		tm_fill_ms(&timeout, 2000);
@@ -1296,9 +1368,8 @@ sleeping_thread(void *arg)
 		tm_now_exact(&start);
 		if (thread_timed_sigsuspend(&oset, &timeout)) {
 			tm_now_exact(&end);
-			printf("%s() suspended %u ms (about 1 sec) before getting signal\n",
+			emit("%s() suspended %u ms (about 1 sec) before getting signal",
 				G_STRFUNC, (uint) tm_elapsed_ms(&end, &start));
-			fflush(stdout);
 		} else {
 			g_assert_not_reached();
 		}
@@ -1315,10 +1386,17 @@ test_signals(void)
 	barrier_t *b;
 	int r, i;
 
-	if (-1 != thread_kill(12, TSIG_0))		/* 12 is random constant */
-		s_error("thread #12 already exists?");
+	TESTING(G_STRFUNC);
+
+	/* 60 is random constant, large enough to avoid it being already used */
+
+	if (-1 != thread_kill(60, TSIG_0))
+		s_error("thread #60 already exists?");
+
+	r = thread_create(signalled_thread, NULL, THREAD_F_PANIC, 0);
+
+	emit("%s() thread %s created", G_STRFUNC, thread_id_name(r));
 
-	r = thread_create(signalled_thread, NULL, 0, 0);
 	for (i = 0; i < 10; i++) {
 		sleep(1);
 		if (-1 == thread_kill(r, TSIG_0))
@@ -1329,32 +1407,41 @@ test_signals(void)
 			s_error("thread #%d cannot be signalled: %m", r);
 	}
 
-	printf("%s() emitting each signal 100 times\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s() emitting each signal 100 times", G_STRFUNC);
 
 	for (i = 0; i < 100; i++) {
-		thread_kill(r, TSIG_1);
-		thread_kill(r, TSIG_2);
-		thread_kill(r, TSIG_4);
+		emitz("%d", i);
+		if (-1 == thread_kill(r, TSIG_1))
+			s_error("cannot send TSIG_1: %m");
+		if (-1 == thread_kill(r, TSIG_2))
+			s_error("cannot send TSIG_2: %m");
+		if (-1 == thread_kill(r, TSIG_4))
+			s_error("cannot send TSIG_4: %m");
 	}
 
-	thread_kill(r, TSIG_3);
-	thread_join(r, NULL);
+	emit("%s() done sending!", G_STRFUNC);
 
-	printf("%s() now checking thread_sleep_ms()\n", G_STRFUNC);
-	fflush(stdout);
+	if (-1 == thread_kill(r, TSIG_3))
+		s_error("cannot send TSIG_3: %m");
+	if (-1 == thread_join(r, NULL))
+		s_error("cannot join: %m");
+
+	emit("%s() now checking thread_sleep_ms()", G_STRFUNC);
 
 	b = barrier_new(2);
-	r = thread_create(sleeping_thread, barrier_refcnt_inc(b), 0, 0);
-	thread_sleep_ms(500);		/* Give it time to setup */
+	r = thread_create(sleeping_thread,
+			barrier_refcnt_inc(b), THREAD_F_PANIC, 0);
+	barrier_wait(b);		/* Give it time to setup */
 	for (i = 0; i < TEST_SIGNALS_COUNT; i++) {
 		if (-1 == thread_kill(r, TSIG_1))
 			s_error("thread #%d cannot be signalled: %m", r);
 		thread_sleep_ms(500);	/* Give it time to process signal */
 	}
+	emit("%s() all signals sent", G_STRFUNC);
 
-	printf("%s() now checking thread_timed_sigsuspend()\n", G_STRFUNC);
-	fflush(stdout);
+	barrier_wait(b);		/* Let signalled thread process everything */
+
+	emit("%s() now checking thread_timed_sigsuspend()", G_STRFUNC);
 
 	barrier_wait(b);
 	thread_kill(r, TSIG_2);
@@ -1382,30 +1469,26 @@ computer_thread(void *arg)
 	barrier_refcnt_inc(cb);
 	g_assert(0 == counter);
 
-	printf("%s(%d) started as %s\n", G_STRFUNC, n, thread_name());
-	fflush(stdout);
+	emit("%s(%d) started as %s", G_STRFUNC, n, thread_name());
 
 	thread_signal(TSIG_1, test_sighandler);
 	WFREE(ca);
 	barrier_wait(cb);
 
-	printf("%s(%d) incrementing counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) incrementing counter=%d", G_STRFUNC, n,
 		atomic_int_get(&counter));
-	fflush(stdout);
 	atomic_int_inc(&counter);
 
 	barrier_wait(cb);
 
-	printf("%s(%d) reincrementing counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) reincrementing counter=%d", G_STRFUNC, n,
 		atomic_int_get(&counter));
-	fflush(stdout);
 	atomic_int_inc(&counter);
 
 	barrier_wait(cb);
 	barrier_free_null(&cb);
 
-	printf("%s(%d) exiting\n", G_STRFUNC, n);
-	fflush(stdout);
+	emit("%s(%d) exiting", G_STRFUNC, n);
 
 	return NULL;
 }
@@ -1425,21 +1508,18 @@ test_barrier_one(bool emulated)
 		WALLOC(ca);
 		ca->n = i;
 		ca->b = cb;
-		t[i] = thread_create(computer_thread, ca, THREAD_F_DETACH, 0);
-		if (-1 == t[i])
-			s_error("cannot create processor thread %u: %m", i);
+		t[i] = thread_create(computer_thread, ca,
+				THREAD_F_DETACH | THREAD_F_PANIC, 0);
 	}
 
 	sleep(1);					/* Wait until threads have started */
 	g_assert(0 == counter);		/* Nobody can change that before the barrier */
 
 	barrier_wait(cb);
-	printf("%s() reached barrier the first time: threads started\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s() reached barrier the first time: threads started", G_STRFUNC);
 
 	barrier_master_wait(cb);
-	printf("%s() reached barrier the second time as master\n", G_STRFUNC);
-	fflush(stdout);
+	emit("%s() reached barrier the second time as master", G_STRFUNC);
 
 	for (i = 0; i < n; i++) {
 		if (-1 == thread_kill(t[i], TSIG_1))
@@ -1451,13 +1531,12 @@ test_barrier_one(bool emulated)
 	atomic_int_inc(&counter);
 	g_assert(n + 1 == counter);	/* We're the master thread */
 
-	printf("%s() releasing threads, counter=%d\n", G_STRFUNC, counter);
-	fflush(stdout);
+	emit("%s() releasing threads, counter=%d", G_STRFUNC, counter);
 	barrier_release(cb);
 
 	barrier_wait(cb);
 	barrier_free_null(&cb);
-	printf("%s() computation done, counter=%d (expected is %d)\n",
+	emit("%s() computation done, counter=%d (expected is %d)",
 		G_STRFUNC, counter, 2 * n  + 1);
 	g_assert(2 * n + 1 == counter);
 }
@@ -1467,6 +1546,8 @@ test_barrier(unsigned repeat, bool emulated)
 {
 	unsigned i;
 
+	TESTING(G_STRFUNC);
+
 	for (i = 0; i < repeat; i++) {
 		test_barrier_one(emulated);
 	}
@@ -1488,38 +1569,33 @@ dam_thread(void *arg)
 	dam_t *d = da->d;
 	barrier_t *b= da->b;
 
-	printf("%s(%d) started as %s\n", G_STRFUNC, n, thread_name());
+	emit("%s(%d) started as %s", G_STRFUNC, n, thread_name());
 
 	dam_wait(d);
 
-	printf("%s(%d) incrementing counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) incrementing counter=%d", G_STRFUNC, n,
 		atomic_int_get(&dam_counter));
-	fflush(stdout);
 	atomic_int_inc(&dam_counter);
 
 	dam_wait(d);
 
-	printf("%s(%d) reincrementing counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) reincrementing counter=%d", G_STRFUNC, n,
 		atomic_int_get(&dam_counter));
-	fflush(stdout);
 	atomic_int_inc(&dam_counter);
 
 	dam_wait(d);		/* Dam disabled, will not wait */
-	printf("%s(%d) last incrementing counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) last incrementing counter=%d", G_STRFUNC, n,
 		atomic_int_get(&dam_counter));
-	fflush(stdout);
 	atomic_int_inc(&dam_counter);
 	dam_free_null(&d);
 
-	printf("%s(%d) waiting, counter=%d\n", G_STRFUNC, n,
+	emit("%s(%d) waiting, counter=%d", G_STRFUNC, n,
 		atomic_int_get(&dam_counter));
-	fflush(stdout);
 
 	barrier_wait(b);
 	barrier_free_null(&b);
 
-	printf("%s(%d) exiting\n", G_STRFUNC, n);
-	fflush(stdout);
+	emit("%s(%d) exiting", G_STRFUNC, n);
 
 	return NULL;
 }
@@ -1543,9 +1619,8 @@ test_dam_one(bool emulated)
 		da->n = i;
 		da->d = dam_refcnt_inc(d);
 		da->b = barrier_refcnt_inc(b);
-		t[i] = thread_create(dam_thread, da, THREAD_F_DETACH, 0);
-		if (-1 == t[i])
-			s_error("cannot create dam thread %u: %m", i);
+		t[i] = thread_create(dam_thread, da,
+				THREAD_F_DETACH | THREAD_F_PANIC, 0);
 	}
 
 	thread_sleep_ms(500);
@@ -1563,6 +1638,8 @@ test_dam(unsigned repeat, bool emulated)
 {
 	unsigned i;
 
+	TESTING(G_STRFUNC);
+
 	for (i = 0; i < repeat; i++) {
 		test_dam_one(emulated);
 	}
@@ -1765,7 +1842,7 @@ exercise_memory(void *arg)
 	er->alloc_us = tm_elapsed_us(&end, &start);
 
 	if (randomize_free)
-		shuffle(mem, filled, sizeof mem[0]);
+		SHUFFLE_ARRAY_N(mem, filled);
 
 	tm_now_exact(&start);
 	for (i = 0; i < filled; i++) {
@@ -1806,10 +1883,8 @@ test_memory_one(struct exercise_results *total, bool posix, int percentage)
 	WALLOC_ARRAY(p, n);
 
 	for (i = 0; i < n; i++) {
-		int r = thread_create(exercise_memory, &ep, 0, THREAD_STACK_MIN);
-		if (-1 == r)
-			s_error("cannot create thread: %m");
-		t[i] = r;
+		t[i] = thread_create(exercise_memory, &ep,
+				THREAD_F_PANIC, THREAD_STACK_MIN);
 
 		if (posix) {
 			pthread_t pt = posix_thread_create(exercise_memory, &ep, TRUE);
@@ -1876,13 +1951,13 @@ test_memory(unsigned repeat, bool posix, int percentage)
 	long cpus = 0 == cpu_count ? getcpucount() : cpu_count;
 	unsigned i;
 
-	printf("%s() detected %ld CPU%s%s\n", G_STRFUNC, cpus, plural(cpus),
+	TESTING(G_STRFUNC);
+
+	emit("%s() detected %ld CPU%s%s", G_STRFUNC, cpus, plural(cpus),
 		0 == cpu_count ? "" : " (forced by -c)");
 
 	if (randomize_free)
-		printf("%s() will free blocks in random order\n", G_STRFUNC);
-
-	fflush(stdout);
+		emit("%s() will free blocks in random order", G_STRFUNC);
 
 	for (i = 0; i < repeat; i++) {
 		tm_t start, end, elapsed;
@@ -1896,16 +1971,13 @@ test_memory(unsigned repeat, bool posix, int percentage)
 
 		tm_elapsed(&elapsed, &end, &start);
 
-		printf("%s() #%d finished! (%f secs, %.3f us/alloc, %.3f us/free)\n",
+		emit("%s() #%d finished! (%f secs, %.3f us/alloc, %.3f us/free)",
 			G_STRFUNC, i, tm2f(&elapsed),
 			total.alloc_us / (double) total.amount,
 			total.free_us / (double) total.amount);
-		fflush(stdout);
 	}
 
-	printf("%s() done!\n", G_STRFUNC);
-	fflush(stdout);
-
+	emit("%s() done!", G_STRFUNC);
 }
 
 static int teq_recv_cnt;
@@ -2028,6 +2100,8 @@ teq_sender(void *arg)
 static void
 test_teq(unsigned repeat)
 {
+	TESTING(G_STRFUNC);
+
 	while (repeat--) {
 		int s, r;
 		barrier_t *b;
@@ -2038,10 +2112,10 @@ test_teq(unsigned repeat)
 
 		b = barrier_new(2);
 		r = thread_create(teq_receiver, barrier_refcnt_inc(b),
-			0, THREAD_STACK_MIN);
+				THREAD_F_PANIC, THREAD_STACK_MIN);
 		arg.receiver = r;
 		arg.b = b;
-		s = thread_create(teq_sender, &arg, 0, THREAD_STACK_MIN);
+		s = thread_create(teq_sender, &arg, THREAD_F_PANIC, THREAD_STACK_MIN);
 
 		thread_join(r, NULL);
 		thread_join(s, NULL);
@@ -2108,11 +2182,13 @@ evq_two(void *unused_arg)
 static void
 test_evq(unsigned repeat)
 {
+	TESTING(G_STRFUNC);
+
 	while (repeat--) {
 		int s, r;
 
-		r = thread_create(evq_one, NULL, 0, THREAD_STACK_MIN);
-		s = thread_create(evq_two, NULL, 0, THREAD_STACK_MIN);
+		r = thread_create(evq_one, NULL, THREAD_F_PANIC, THREAD_STACK_MIN);
+		s = thread_create(evq_two, NULL, THREAD_F_PANIC, THREAD_STACK_MIN);
 
 		thread_join(r, NULL);
 		thread_join(s, NULL);
@@ -2147,11 +2223,12 @@ main(int argc, char **argv)
 	bool signals = FALSE, barrier = FALSE, overflow = FALSE, memory = FALSE;
 	bool stats = FALSE, teq = FALSE, cancel = FALSE, dam = FALSE, evq = FALSE;
 	unsigned repeat = 1, play_time = 0;
-	const char options[] = "a:b:c:ef:hjn:r:st:vwxABCDEFIKMNOPQRST:VWX";
+	const char options[] = "a:b:c:ef:hjn:r:st:vwxz:ABCDEFIKMNOPQRST:VWX";
 
 	mingw_early_init();
 	progname = filepath_basename(argv[0]);
 	thread_set_main(TRUE);		/* We're the main thread, we can block */
+	crash_init(argv[0], progname, 0, NULL);
 	stacktrace_init(argv[0], FALSE);
 
 	misc_init();
@@ -2255,6 +2332,9 @@ main(int argc, char **argv)
 		case 'x':			/* free allocated memory by -X tests randomly */
 			randomize_free = TRUE;
 			break;
+		case 'z':			/* zap message from routines using emitz() */
+			zap_record(optarg);
+			break;
 		case 'h':			/* show help */
 		default:
 			usage();
@@ -2323,16 +2403,16 @@ main(int argc, char **argv)
 		case 'r':
 			break;
 		case 'h':
-			printf("Using halloc() for memory tests\n");
+			emit("Using halloc() for memory tests");
 			break;
 		case 'v':
-			printf("Using vmm_alloc() for memory tests\n");
+			emit("Using vmm_alloc() for memory tests");
 			break;
 		case 'w':
-			printf("Using walloc() for memory tests\n");
+			emit("Using walloc() for memory tests");
 			break;
 		case 'x':
-			printf("Using xmalloc() for memory tests\n");
+			emit("Using xmalloc() for memory tests");
 			break;
 		default:
 			s_warning("unknown allocator '%c', using random mix", allocator);
@@ -2340,14 +2420,13 @@ main(int argc, char **argv)
 			break;
 		}
 		if (allocator_bsize != 0) {
-			printf("Using blocks of %lu byte%s\n", (ulong) allocator_bsize,
+			emit("Using blocks of %lu byte%s", (ulong) allocator_bsize,
 				plural(allocator_bsize));
 		}
 		if (posix)
-			printf("Adding (discovered) POSIX threads\n");
+			emit("Adding (discovered) POSIX threads");
 		if (percentage)
-			printf("Randomly free %d%% blocks in remote threads\n", percentage);
-		fflush(stdout);
+			emit("Randomly free %d%% blocks in remote threads", percentage);
 		test_memory(repeat, posix, percentage);
 	}
 
@@ -2390,3 +2469,4 @@ main(int argc, char **argv)
 	exit(EXIT_SUCCESS);	/* Required to cleanup semaphores if not destroyed */
 }
 
+/* vi: set ts=4 sw=4 cindent: */
