@@ -456,13 +456,8 @@ tm_thread_main(void *unused_arg)
 static void
 tm_thread_start(void)
 {
-	int r;
-
-	r = thread_create(tm_thread_main, NULL,
-			THREAD_F_DETACH | THREAD_F_NO_POOL, TM_THREAD_STACK);
-
-	if G_UNLIKELY(-1 == r)
-		s_minierror("%s(): cannot launch time thread: %m", G_STRFUNC);
+	thread_create(tm_thread_main, NULL,
+		THREAD_F_DETACH | THREAD_F_NO_POOL | THREAD_F_PANIC, TM_THREAD_STACK);
 }
 
 /**
@@ -525,10 +520,8 @@ int
 tm_cmp(const tm_t *a, const tm_t *b)
 {
 	if (a->tv_sec != b->tv_sec)
-		return (a->tv_sec > b->tv_sec) ? +1 : -1;
-	if (a->tv_usec == b->tv_usec)
-		return 0;
-	return (a->tv_usec > b->tv_usec) ? +1 : -1;
+		return CMP(a->tv_sec, b->tv_sec);
+	return CMP(a->tv_usec, b->tv_usec);
 }
 
 /**
@@ -577,7 +570,15 @@ tm_remaining_ms(const tm_t *end)
 	tm_t now, elapsed;
 	long remain;
 
-	tm_now_exact(&now);
+	/*
+	 * This routine is usually called from low-level code and we do not
+	 * want to dispatch signals or suspend the calling thread at this stage
+	 * since this will usually be done afterwards in the same low-level
+	 * calling routine.
+	 *		--RAM, 2015-02-25
+	 */
+
+	tm_now_exact_raw(&now);			/* Raw version, no signals nor suspension */
 	tm_elapsed(&elapsed, end, &now);
 	remain = tm2ms(&elapsed);
 
@@ -612,6 +613,24 @@ tm_now(tm_t *tm)
 	}
 }
 
+/**
+ * Fill supplied structure with current time (recomputed).
+ *
+ * @attention
+ * This raw version does not check for thread suspension.  It is meant
+ * to be used in dire circumstances to limit resource consumption or
+ * avoid any suspension or signal delivery.
+ */
+void
+tm_now_exact_raw(tm_t *tm)
+{
+	TM_LOCK;
+	tm_current_time(&tm_cached_now);
+	if G_LIKELY(tm != NULL)
+		*tm = tm_cached_now;
+	TM_UNLOCK;
+}
+
 /**
  * Fill supplied structure with current time (recomputed).
  */
@@ -636,12 +655,7 @@ tm_now_exact(tm_t *tm)
 	}
 
 	thread_check_suspended();
-
-	TM_LOCK;
-	tm_current_time(&tm_cached_now);
-	if G_LIKELY(tm != NULL)
-		*tm = tm_cached_now;
-	TM_UNLOCK;
+	tm_now_exact_raw(tm);
 }
 
 /**
