@@ -627,8 +627,8 @@ tmalloc_depot_return_empty(tmalloc_t *d, tmalloc_magazine_t *m)
 			free_magazine = TRUE;
 		else
 			eslist_prepend(&d->tma_empty.tml_list, m);
-	} else {
-		d->tma_magazines++;		/* Going to return empty magazine to thread */
+
+		d->tma_magazines--;		/* Thread returned a magazine */
 	}
 
 	/*
@@ -662,12 +662,12 @@ tmalloc_depot_return_empty(tmalloc_t *d, tmalloc_magazine_t *m)
 		}
 	}
 
-	tmalloc_depot_unlock_hidden(d);
-
-	if G_LIKELY(fm != NULL)
+	if G_LIKELY(fm != NULL) {
 		TMALLOC_STATS_INCX(d, mag_full_loaded);
-	else
-		d->tma_magazines--;		/* No magazine returned */
+		d->tma_magazines++;			/* Returning magazine to thread */
+	}
+
+	tmalloc_depot_unlock_hidden(d);
 
 	if G_UNLIKELY(free_magazine) {
 		TMALLOC_STATS_INCX(d, mag_bad_capacity);
@@ -713,7 +713,7 @@ tmalloc_depot_return_full(tmalloc_t *d, tmalloc_magazine_t *m)
 		else
 			eslist_prepend(&d->tma_full.tml_list, m);
 	} else {
-		d->tma_magazines++;		/* Going to return full magazine to thread */
+		d->tma_magazines++;		/* We always return a new magazine */
 	}
 
 	tmalloc_depot_unlock_hidden(d);
@@ -953,7 +953,11 @@ tmalloc_thread_layer_free(void *data)
 	tmalloc_check(d);
 
 	if (tmalloc_debugging(10)) {
-		s_debug("%s(\"%s\"): %s is exiting",
+		/*
+		 * Use s_rawdebug() to avoid memory allocation on the thread exit path.
+		 */
+
+		s_rawdebug("%s(\"%s\"): %s is exiting",
 			G_STRFUNC, d->tma_name, thread_name());
 	}
 
@@ -969,6 +973,21 @@ tmalloc_thread_layer_free(void *data)
 
 	tmt->tmt_magic = 0;
 	d->tma_free(tmt, sizeof *tmt);
+
+	/*
+	 * There is no need to remove the now destroyed "tmt" variable from
+	 * the "tmagazines" list: that list is only maintained to quickly
+	 * access all the "struct tmalloc_thread" objects created for a thread
+	 * when garbage-collecting the unused magazines.
+	 *
+	 * The embedded list descriptor itself is held in a thread-local variable
+	 * and will be cleared by tmalloc_thread_free_magazines().
+	 *
+	 * Al the "struct tmalloc_thread" objects pertaining to the dying thread
+	 * are therefore reclaimed not because they are part of the "tmagazines"
+	 * list, but because they are indexed via a specific thread-local variable:
+	 * the tma_key in the tmalloc_t object (the magazine depot).
+	 */
 }
 
 static void
@@ -1438,7 +1457,7 @@ tmalloc_thread_gc(void *unused_data)
 	 * If any thread layer has not been performing any operation for the
 	 * last TMALLOC_TGC_IDLE seconds, then clear its magazines to avoid
 	 * keeping objects allocated in the magazines that never get used.  This
-	 * s especially important for the larger objects, or for large magazines.
+	 * is especially important for the larger objects, or for large magazines.
 	 */
 
 	ESLIST_FOREACH_DATA(tmagazines, tmt) {
@@ -1496,7 +1515,7 @@ tmalloc_beat(void *data)
 	if (
 		elapsed >= TMALLOC_BEAT_THRESHOLD ||
 		(elapsed > 0 &&
-			d->tma_contentions / elapsed > (int) 5 * TMALLOC_CONTENTIONS)
+			d->tma_contentions / elapsed > (int) (5 * TMALLOC_CONTENTIONS))
 	) {
 		size_t contentions;
 		double rate;
@@ -1991,7 +2010,7 @@ tmalloc_thread_create(tmalloc_t *tma)
 		 * first attempt at using any thread magazine alllocator will create
 		 * the magazine list, entering this "if" statement.
 		 *
-		 * Becasue we register that callback after tmalloc_thread_gc_install(),
+		 * Because we register that callback after tmalloc_thread_gc_install(),
 		 * we know that it will be run before any exiting callback used by
 		 * the event queue (execution order is LIFO).
 		 */
