@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2001-2011, Raphael Manfredi
- * Copyright (c) 2005-2011, Christian Biere
+ * Copyright (c) 2001-2015 Raphael Manfredi
+ * Copyright (c) 2005-2011 Christian Biere
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -29,7 +29,7 @@
  * Main functions for gtk-gnutella.
  *
  * @author Raphael Manfredi
- * @date 2001-2011
+ * @date 2001-2015
  * @author Christian Biere
  * @date 2005-2011
  */
@@ -116,6 +116,7 @@
 #include "lib/crc.h"
 #include "lib/dbus_util.h"
 #include "lib/debug.h"
+#include "lib/entropy.h"
 #include "lib/eval.h"
 #include "lib/evq.h"
 #include "lib/exit.h"
@@ -417,6 +418,18 @@ gtk_gnutella_request_shutdown(enum shutdown_mode mode, unsigned flags)
 	shutdown_user_flags = flags;
 }
 
+/**
+ * Crash restart callback to trigger a graceful asynchronous restart.
+ */
+static int
+gtk_gnutella_request_restart(void)
+{
+	gtk_gnutella_request_shutdown(GTKG_SHUTDOWN_NORMAL,
+		GTKG_SHUTDOWN_OFAST | GTKG_SHUTDOWN_ORESTART | GTKG_SHUTDOWN_OCRASH);
+
+	return 1;	/* Any non-zero return signifies: async restart in progress */
+}
+
 /**
  * Manually dispatch events that are normally done from glib's main loop.
  */
@@ -453,7 +466,7 @@ main_dispatch(void)
 /*
  * If they requested abnormal termination after shutdown, comply now.
  */
-static G_GNUC_COLD void
+static void G_GNUC_COLD
 handle_user_shutdown_request(enum shutdown_mode mode)
 {
 	const char *msg = "crashing at your request";
@@ -491,13 +504,24 @@ handle_user_shutdown_request(enum shutdown_mode mode)
  * Shutdown systems, so we can track memory leaks, and wait for EXIT_GRACE
  * seconds so that BYE messages can be sent to other nodes.
  */
-G_GNUC_COLD void
+void G_GNUC_COLD
 gtk_gnutella_exit(int exit_code)
 {
 	static volatile sig_atomic_t safe_to_exit;
 	time_t exit_time = time(NULL);
 	time_delta_t exit_grace = EXIT_GRACE;
-	bool byeall = TRUE;
+	bool byeall =
+		!(shutdown_requested && (shutdown_user_flags & GTKG_SHUTDOWN_OFAST));
+	bool crashing =
+		shutdown_requested && (shutdown_user_flags & GTKG_SHUTDOWN_OCRASH);
+
+	/*
+	 * In case this routine is part of an automatic restarting sequence,
+	 * signal to the crashing layer that we are starting in order to cancel
+	 * the automatic restart after some time.
+	 */
+
+	crash_restarting();
 
 	if (exiting) {
 		if (safe_to_exit) {
@@ -509,9 +533,6 @@ gtk_gnutella_exit(int exit_code)
 		return;
 	}
 
-	if (shutdown_requested && (shutdown_user_flags & GTKG_SHUTDOWN_OFAST))
-		byeall = FALSE;
-
 	exiting = TRUE;
 
 #define DO(fn) 	do {					\
@@ -530,6 +551,7 @@ gtk_gnutella_exit(int exit_code)
 	fn(arg);								\
 } while (0)
 
+	DO(socket_shutdowning);			/* We're about to shutdown for good */
 	DO(shell_close);
 	DO(file_info_store_if_dirty);	/* For safety, will run again below */
 	DO(file_info_close_pre);
@@ -578,6 +600,7 @@ gtk_gnutella_exit(int exit_code)
 	 */
 
 	if (debugging(0)) {
+		DO(random_dump_stats);
 		DO(palloc_dump_stats);
 		DO(tmalloc_dump_stats);
 		DO(vmm_dump_stats);
@@ -649,6 +672,19 @@ gtk_gnutella_exit(int exit_code)
 		log_cpu_usage(&since, NULL, NULL);
 	}
 
+	/*
+	 * Skip gracetime for BYE message to go through when crashing, as well
+	 * as most the final exit sequence whose aim is to properly clean memory
+	 * to be able to trace leaks when debugging.
+	 */
+
+	if (crashing) {
+		/* Accelerated shutdown */
+		DO(settings_close);
+		DO(cq_close);
+		goto quick_restart;
+	}
+
 	/*
 	 * Wait at most EXIT_GRACE seconds, so that BYE messages can go through.
 	 * This amount of time is doubled when running in Ultra mode since we
@@ -859,7 +895,8 @@ gtk_gnutella_exit(int exit_code)
 	DO(vmm_close);
 	DO(signal_close);
 
-	g_info("gtk-gnutella shut down cleanly.");
+quick_restart:
+	g_info("gtk-gnutella shut down %s.", crashing ? "quickly" : "cleanly");
 
 	if (shutdown_requested) {
 		handle_user_shutdown_request(shutdown_user_mode);
@@ -1044,7 +1081,7 @@ option_pretty_name(const char *name)
 	return buf;
 }
 
-static void
+static void G_GNUC_NORETURN
 usage(int exit_code)
 {
 	FILE *f;
@@ -1283,6 +1320,19 @@ slow_main_timer(time_t now)
 	download_slow_timer(now);
 	node_slow_timer(now);
 	ignore_timer(now);
+
+	/*
+	 * Unfortunately, Linux is not always correctly handling SO_REUSEADDR:
+	 * a bind() can fail with errno set to EADDRINUSE despite the flag being
+	 * requested on the socket descriptor prior to calling bind().
+	 *
+	 * So we have no other choice but to periodically re-attempt the creation
+	 * when we end-up with no TCP listening socket.
+	 *		--RAM, 2015-05-17
+	 */
+
+	if (GNET_PROPERTY(tcp_no_listening))
+		settings_create_listening_sockets();
 }
 
 /**
@@ -1479,6 +1529,8 @@ main_timer(void *unused_data)
 typedef void (*digest_collector_cb_t)(sha1_t *digest);
 
 static digest_collector_cb_t random_source[] = {
+	random_stats_digest,
+	halloc_stats_digest,
 	palloc_stats_digest,
 	gnet_stats_tcp_digest,
 	thread_stats_digest,
@@ -1488,6 +1540,7 @@ static digest_collector_cb_t random_source[] = {
 	tmalloc_stats_digest,
 	xmalloc_stats_digest,
 	zalloc_stats_digest,
+	entropy_minimal_collect,
 };
 
 /**
@@ -1496,16 +1549,17 @@ static digest_collector_cb_t random_source[] = {
 static bool
 callout_queue_idle(void *unused_data)
 {
-	bool overloaded = GNET_PROPERTY(overloaded_cpu);
 	sha1_t digest;
 	static uint ridx = 0;
 	static size_t counter = 0;
 
 	(void) unused_data;
 
-	if (GNET_PROPERTY(cq_debug) > 1)
+	if (GNET_PROPERTY(cq_debug) > 1) {
+		bool overloaded = GNET_PROPERTY(overloaded_cpu);
 		g_debug("CQ: callout queue is idle (CPU %s)",
 			overloaded ? "OVERLOADED" : "available");
+	}
 
 	/* Idle tasks always scheduled */
 	random_collect();
@@ -1870,6 +1924,7 @@ main(int argc, char **argv)
 			product_get_patchlevel());
 		crash_setbuild(product_get_build());
 		crash_setmain(main_argc, main_argv, main_env);
+		crash_set_restart(gtk_gnutella_request_restart);
 	}	
 	stacktrace_init(argv[0], TRUE);	/* Defer loading until needed */
 	handle_arguments_asap();
